
import ailib
import ailib.fitting
from math import log, e, exp
from models import *
from scipy import matrix

def writeData(data, fName='train.t'):
    f = open(fName, 'w')
    for (x,y) in data:
        f.write(str(x) + '\t' + str(y) + '\n')
    f.close()

def vote(items):
    x = ailib.mean(map(ailib.fst, items))
    y = ailib.mean(map(ailib.snd, items))
    return (x,y)

if __name__ == '__main__':
    
    # read
    d = [map(float,l.split()) for l in file('expResult4042cropped.g')]
    
    # crop
    #d = filter(lambda (x,y): x < 2400 and x > 1700 and y < 1300 and y > 1150, d) # left page
    #d = filter(lambda (x,y): x < 3000 and x > 2400 and y < 1300 and y > 1150, d) # right page
    #d = filter(lambda (x,y): x < 3000 and x > 1700 and y < 1300 and y > 1150, d)  # both pages
    
    # find characteristics
    xLo = min(map(ailib.fst,d))
    xHi = max(map(ailib.fst,d))
    xSp = ailib.span(map(ailib.fst,d))
    yLo = min(map(ailib.snd,d))
    ySp = ailib.span(map(ailib.snd,d))
    
    m = map(lambda (x,y): (xHi-x, y), d)
    
    # mirror data, such that they are in the first quadrant
    
    #n = map(lambda (x,y): (10.0+4.0*(xHi-x)/xSp, 2.0*(y-yLo)/ySp), d)
    n = map(lambda (x,y): (2.0 * (x-xLo)/xSp-1, (y-yLo)/ySp), d)
    
    # reduce
    r = map(vote, ailib.linsplit(n, 5, key=ailib.fst))
    
    writeData(n, 'train.t')
    writeData(r, 'red.t')
    
    # do fitting
    def gen(d, f, init=None):
        if init is None: init = f.init
        samples, labels = ailib.Format.short.unpack(d)
        m = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
        m.fit(samples, labels, weights=None, theta=None, eps0=None, eps1=None, tau=None)
    
        # output
        print 'f(x) =', f.str(m)
    
    def bigen(d, f, t=(1-0.41), init=None):
        if init is None: init = f.init
        
        n0 = filter(lambda (x,y): x<t, d)
        n1 = filter(lambda (x,y): x>=t, d)
        
        samples0, labels0 = ailib.Format.short.unpack(n0)
        samples1, labels1 = ailib.Format.short.unpack(n1)
        
        samples0 = map(lambda x: t-x, samples0)
        samples1 = map(lambda x: x-t, samples1)
        
        m = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
        m.fit(samples0, labels0, weights=None, theta=None, eps0=None, eps1=None, tau=None)
        print 'f(x) =', f.str(m)
        
        m.fit(samples1, labels1, weights=None, theta=None, eps0=None, eps1=None, tau=None)
        print 'g(x) =', f.str(m)

    def thresFitting(d, f, init=None, (t,t_max,t_step)=(-0.5, 0.5, 0.1)):
        if init is None: init = f.init
        samples, labels = ailib.Format.short.unpack(n)
        m0 = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
        m1 = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
        
        err_best, best = float('inf'), None
        
        while t < t_max:
            # left
            left = map(lambda (x,y): (t-x,y), filter(lambda (x,y): x < t, d))
            samples, labels = ailib.Format.short.unpack(left)
            m0.fit(samples, labels, weights=None, theta=None, eps0=None, eps1=None, tau=None)
            #print 'f(x) =', f.str(m)
            
            # right
            right = map(lambda (x,y): (x-t,y), filter(lambda (x,y): x >= t, d))
            samples, labels = ailib.Format.short.unpack(right)
            m1.fit(samples, labels, weights=None, theta=None, eps0=None, eps1=None, tau=None)
            #print 'g(x) =', f.str(m)


            err = sum([m0.err(s) for s in left]) + sum([m1.err(s) for s in right])
            print "Error", t, '\t', err
            
            if err < err_best:
                err_best = err
                best = (m0.coeff[:], m1.coeff[:])
                print "> Best", t
            
            t += t_step
        
        if best is not None: m0.coeff, m1.coeff = best
        return m0, m1
