
import ailib
import ailib.fitting
from math import log, e, exp
from models import *
from scipy import matrix

def readData(fName):
    f = open(fName, 'r')
    f.close()

def writeData(data, fName='train.t'):
    f = open(fName, 'w')
    for (x,y) in data:
        f.write(str(x) + '\t' + str(y) + '\n')
    f.close()

def writePlot(m0, m1, data, fName=''):
	f = open(fName, 'w')
	f.close()

def normalize(d):
    """Arrange the data *d* around the origin and dimension [2x1]."""
    xLo = min(map(ailib.fst,d))
    yLo = min(map(ailib.snd,d))
    xSp = ailib.span(map(ailib.fst,d))
    ySp = ailib.span(map(ailib.snd,d))
    n = map(lambda (x,y): (2.0 * (x-xLo)/xSp-1, (y-yLo)/ySp), d)
    return n

def thresFitting(d, f, init=None, (t,t_max,t_step)=(-0.5, 0.5, 0.1)):
    """
    Execute Marquardt NLSQ on the function *f* with data *d*. The data
    is split at a threshold, each part is fitted individually (to the same
    function *f*). The threshold is moved in [*t*, *t_max*] with step size *t_step*.
    The models with minimal overall error is returned.
    """
    
    # init NLSQ with defaults
    if init is None: init = f.init
    samples, labels = ailib.Format.short.unpack(d)
    m0 = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
    m1 = ailib.fitting.Marquardt(f.model, f.partials, initial=init, eps0=0.01, eps1=0.01, tau=1.0, maxIter=200)
    
    err_best, best = float('inf'), None
    
    best_t = t
    while t < t_max:
        # left
        left = map(lambda (x,y): (t-x,y), filter(lambda (x,y): x < t, d))
        samples, labels = ailib.Format.short.unpack(left)
        m0.fit(samples, labels, weights=None, theta=None, eps0=None, eps1=None, tau=None)
        #print 'f(x) =', f.str(m)
        
        # right
        right = map(lambda (x,y): (x-t,y), filter(lambda (x,y): x >= t, d))
        samples, labels = ailib.Format.short.unpack(right)
        m1.fit(samples, labels, weights=None, theta=None, eps0=None, eps1=None, tau=None)
        #print 'g(x) =', f.str(m)
        
        
        err = sum([m0.err(s) for s in left]) + sum([m1.err(s) for s in right])
        print "Error", t, '\t', err
        
        if err < err_best:
            err_best = err
            best = (m0.coeff[:], m1.coeff[:])
	    best_t = t
            print "> Best", t
        
        t += t_step
    
    if best is not None: m0.coeff, m1.coeff = best
    return m0, m1, best_t


if __name__ == '__main__':

    # read data
    
    # model fitting
    f = Func2()
    m0, m1 = thresFitting(data, f, None, (-0.5, 0.5, 0.1))