
import Image

def getProfile(fName, tol=5, inv=False):
	"""Get the profile from a black-white image located at *fName*
	by searching from top to bottom. Get one point for all values
	in the x direction. A profile point is the first point (starting
	from the top) that has a value below *tol*.
	
	Return a list of y coordinates (measured from top) for each x
	coordinate (starting at zero).
	
	If *inv* is True, the y coordinate is measured from the bottom.
	"""
	src = Image.open(fName)
	img = src.load()
	W,H = src.size
	
	profile = []
	for x in range(W): # left to right
		for y in xrange(H): # top to bottom
			if img[x,y] <= tol: break
		profile.append( y )
	
	if inv: profile = map(lambda y: H-y, profile)
	return profile

if __name__ == '__main__':
	fName = 'test/trd/profile0001_edit.jpg'
	p = getProfile(fName, 10.0)
	#print p