
from motionTrigger import *

class TriggerCV(MotionTrigger):
    def __init__(self, capture, callback, windowSize=5, thres=None):
        MotionTrigger.__init__(self, capture, callback, windowSize, thres)
        self.history = []
        
        # create windows
        cv.NamedWindow('Camera')
        print "Press ESC to quit."

    def _post(self, frm, curr, diff, motion, trigger):
        self.history.append( (frm, diff, motion) )
        
        # display webcam image
        cv.ShowImage('Camera', self.capture.last)

        # handle events
        k = cv.WaitKey(40) & 255
        if k in (27,113):
            print 'ESC pressed. Exiting ...'
            self.running = False


__thres__ = 35000000                # Threshold for difference
__autothres__ = True                # Adjust the threshold automatically.
__autothres_scale__ = 1.5           # Auto threshold scale factor
__windowSize__ = 3                  # Majority vote window
__defaultDevice__ = 0               # Input camera device number
__skip__ = 20                       # Number of frames to skip before motion detection
__width__, __height__ = 640, 480    # Dimensions of camera images
__timeout__ = 3                     # Number of frames to skip between measurements
__distfile__ = '/tmp/dst.t'         # Distance information target file. None for no output
__motionfile__ = '/tmp/mtn.t'       # Motion information target file. None for no output
__maxframe__ = -1                   # Maximal number of measurements. Negative for infinite loop

if __name__ == "__main__":
    
    # init capture device
    device = len(sys.argv) == 2 and int(sys.argv[1]) or __defaultDevice__ # use argument or assume we want first device
    capture = CaptureCV(device, __width__, __height__, __skip__, __timeout__)
    
    # define callback
    callback = lambda frame_no, diff, motion: sys.stdout.write("Trigger\n")
    
    # init trigger
    trig = TriggerCV(capture, callback, __windowSize__, __thres__)
    if __autothres__:
        sys.stdout.write('Auto adjusting threshold... ')
        sys.stdout.flush()
        trig.thres = trig.autothres(2 * __skip__, __autothres_scale__)
        print "set to", trig.thres
    
    # run
    trig.run(__maxframe__)

    # write output file
    #__distfile__, __motionfile__ = None, None
    if __distfile__ is not None:
        f = open(__distfile__, 'w')
        for (i,d,m) in trig.history: f.write(str(i) + '\t' + str(d) + '\n')
        f.close()

    if __motionfile__ is not None:
        g = open(__motionfile__, 'w')
        for (i,d,m) in trig.history: g.write(str(i) + '\t' + str(trig.thres * 2 * int(m)) + '\n')
        g.close()

    print "plot '" + __motionfile__ + "' with lines, '" + __distfile__ + "' with lines, " + str(trig.thres)
