
import include
import re, copy, sys

from Line import Line
from Shape import Point, Square
from custImage import custImage
import mCalc2



class Crop(Square):

	borderParams = {'left': ('H', False), 'right': ('H', True), 'top': ('V', False), 'bottom': ('V', True)}

	def __init__(self, image, metric, elThres=0.5, valThres=127, chances=1):
		self.elThres = elThres
		self.chances = chances
		self.valThres = valThres
		
		self.image = image
		self.metric = metric
		
		if self.image.transform != None: self.initByTrans(self.image.transform, (self.image.width, self.image.height))
		else: Square.__init__(self, Point(0, 0), Point(self.image.width, self.image.height))

	def fromConfig(self, cfg):
		for key in ('elThres', 'chances', 'valThres'):
			try: self.__dict__[key] = cfg[key]
			except KeyError: pass
		return self

	def initByTrans(self, trans, dim):
		t = trans.invert()
		pts = [Point(x, y).transform(t) for x in (0, dim[0]) for y in (0, dim[1])]
		x0, y0 = min(map(lambda p: p.x, pts)), min(map(lambda p: p.y, pts))
		x1, y1 = max(map(lambda p: p.x, pts)), max(map(lambda p: p.y, pts))
		Square.__init__(self, Point(x0, y0), Point(x1, y1))
		return self

	def __readTerm(self, s):
		# (x1.x + 24, 0) + x2, x2
		bcnt=0
		i=0
		while i < len(s):
			if s[i] == ',' and bcnt==0:	break
			elif s[i] == '(':		bcnt += 1
			elif s[i] == ')':		bcnt -= 1
			i += 1
		if i == len(s): raise Exception('parse failure')
		ptA = self.metric.parse(s[0:i].strip())
		ptB = self.metric.parse(s[i+1:].strip())
		return Square(ptA, ptB)

	def __readParam(self, s):
		width = 1
		step = 1
		for p in s.split(','):
			m = re.search('([sw])=(\d+)', p)
			if m == None: break
			elif m.groups()[0] == 'w': width = m[1]
			elif m.groups()[0] == 's': step = m[1]
		return (step, width)


	def fromString(self, border, args):
		method = args[0].upper().strip()
		#print border, method
		if method == 'S':
			# params & defaults
			direction, reverse = self.borderParams[border]
			step, width = (1, 1)
			self.metric.setFixSquare(self, 'c')
			region = self.__readTerm(args[1].strip().lower())
			
			# parse algorithm arguments
			if len(args) > 2:	step, width = self.__readParam(args[2].strip().lower())
			else:			(step, width) = (1, 1)
			
			# region: self.final, mod by args
			line = Line(region, step, width, direction, reverse)
			bestSquare = self.__searchLine(line)
			
			# write back
			if bestSquare != None:
				self.__setBorder(self, border, self.__getBorder(bestSquare, border))
			
		elif method == 'F':
			if len(args) > 1:
				curr = self.metric.parse(args[1])
				if border in ('left', 'right'): self.__setBorder(self, border, curr.x)
				else: self.__setBorder(self, border, curr.y)

	def __searchLine(self, line):
		#print "Scan Line:", line, line.direction, line.reverse
		final = None
		chances = self.chances
		for subLine in line:
			#sys.stdout.write("Scanning " + str(subLine))
			foundCnt = 0
			for shape in subLine:
				value = self.image.getColorFromShape(shape)
				if value > self.valThres:
					foundCnt += 1
			
			if float(foundCnt) / subLine.count() >= self.elThres:
				#print " -> VALID"
				final = subLine
				chances -= 1
				if chances <= 0: break
			else:
				pass
				#print ""
		
		return final

	def __getattr__(self, attr):
		r = self.__getBorder(self, attr)
		if r == False:	return Square.__getattr__(self, attr)
		else:		return r

	def __setattr__(self, attr, value):
		r = self.__setBorder(self, attr, value)
		if r == False: self.__dict__[attr] = value

	def __getBorder(self, obj, name):
		if   name == 'left':	return obj.leftTop.x
		elif name == 'right':	return obj.rightBottom.x
		elif name == 'top':	return obj.leftTop.y
		elif name == 'bottom':	return obj.rightBottom.y
		return False

	def __setBorder(self, obj, name, value):
		if   name == 'left':
			obj.leftTop.x = value
			return True
		elif name == 'right':
			obj.rightBottom.x = value
			return True
		elif name == 'top':
			obj.leftTop.y = value
			return True
		elif name == 'bottom':
			obj.rightBottom.y = value
			return True
		return False



class App:

	def __init__(self):
		# defaults
		self.channel = 'value'
		self.output = sys.stdout
		self.imgPath = ''
		
		# intrinsics
		self.croppings = []
		self.initial = None

	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self

	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['i=','o=','c=','h'], ["image=", "output", "channel=", "help", 'left=', 'right=', 'bottom=', 'top='])
			for opt, arg in opts:
				if opt in ('-i', '--image'):
					self.imgPath = arg
				elif opt in ('-o', '--output'):
					self.output = open(arg, 'w')
				elif opt in ('-c', '--channel'):
					self.channel = arg
				elif opt in ('-h', '--help'):
					self.helpAndExit(0);
				elif opt in ('--left', '--right', '--bottom', '--top'):
					params = arg.split('/')
					self.croppings.append((opt[2:], params))
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		self.image = custImage(self.imgPath, self.channel)
		self.metrics = mCalc2.App().parseArgs(self.rest).initialize()
		self.image.transform = self.metrics.transform
		self.final = Crop(self.image, self.metrics.calc)
		return self


	def execute(self):
		for border, args in self.croppings:
			self.final.fromString(border, args)
		return self


	def printOut(self):
		if self.image.transform != None: fin = self.final.transform(self.image.transform)
		else: fin = self.final
		s = "%ix%i+%i+%i\n" % (int(fin.width), int(fin.height), int(fin.leftTop.x), int(fin.leftTop.y))
		self.output.write(s + '\n')
		return self

	def helpAndExit(self, code=0):
		print "This script crops an image"
		print ""
		print "Usage:"
		print "Paths"
		print "-i --image		source image path"
		print "-o --output		output file path (default stdout)"
		print ""
		print "Probe:"
		print "-c --channel		image channel"
		print "--left --right		Border information. It's specified as"
		print "--top --bottom		S/<left>,<right>,<top>,<bottom>/args or"
		print "			F/<pos>/<args>"
		print "			The data format of the <..> parts is the"
		print "			same as in mcalc (without the [HV]/)"
		print "Reference"
		print "-g --geometry		Image geometry in [mm]"
		print "-d --dimension		Image geometry in [pix]"
		print "-i --image		Image as path (will automatically ex -d)"
		print ""
		print "Transformation"
		print "-p --point		Transformation point"
		print "			There have to be at least 2 points given"
		print "Help"
		print "-h --help		This help"
		print ""
		print "The output is the remaining square (to be cropped)"
		print "in the format [(x0, y0), (x1, y1)]"
		exit(code)

# python -i crop.py -i /home/matthias/Desktop/ctest2/ex1/OUT_0415_Final.jpg -c value --bottom "F/x1 + 138" --left "S/x1 + 24" --top "S/-,-,x0 + 96" --right "S/-" -g 800x1143 -d 280x400 -p 453x219 -p 1248x1323
# [(1310.0, 392.0), (2857.0, 1461.0)]
# 1547x1069+1310+392

# python Crop2.py -i /home/matthias/Desktop/ctest1/OUT_0415_Final.jpg -c value -g 280x400 -d 280x400 -p 453x219 -p 1248x1323 --bottom 'F/x2 + (0, 47)' --left 'S/(x1.x+10, 0), c2' --top 'S/c0 + (0, x1.y), c2' --right 'S/c0, c2'
if __name__ == '__main__': App().run()
