
import include, multiRef, colorReference, colMatch
from custImage import custImage
from Crop2 import Crop
from ColorTrans import ColorTrans
from IMExec import IMExec
from Transform import Transform
from Shape import Point


class Workstation:
	def __init__(self):
		pass

	def fromConfig(self, cfg):
		self.refTrans = multiRef.multiRef().fromConfig(cfg)
		self.metric = self.refTrans.virtLayout.metric
		self.colTrans = colorReference.colorReference().fromConfig(cfg['colorReference'], self.metric)
		self.cropCfg = cfg['cropping']
		return self

	def positionTransform(self, image):
		# get transformation
		transMatrix, scale = self.refTrans.multiRef(image)
		
		# set on image
		image.setTransform(Transform().initByMatrix(transMatrix))
		
		print scale
		return transMatrix, scale

	def colorTransform(self, image):
		cTrans = ColorTrans()
		for channel in ['RED', 'GREEN', 'BLUE']:
			image.channel = channel
			self.colTrans.img.channel = channel
			
			cm = colMatch.colMatch(image, True, True)
			col = cm.matchReference(self.colTrans)
			cTrans.setTrans(channel, col.toCoeffPoly(self.colTrans.polyDeg))
		
		image.colTransform = cTrans
		return cTrans

	def cropping(self, image, metric=None):
		if metric == None: metric = self.metric
		
		image.channel = 'VALUE'
		cr = Crop(image, metric).fromConfig(self.cropCfg)
		
		# crop
		for border, method, query in self.cropCfg['borders']:
			if border not in ('left', 'right', 'top', 'bottom'): continue
			cr.fromString(border, (method, query))
		
		return cr

class App:

	def __init__(self):
		# INTERNALS
		self.debug = False
		self.verbose = False
		self.doColMatch = True
		self.doCrop = True
		self.outPath = ''

		# OUTPUT VARIABLES
		self.imParams = []


	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self


	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['c=', 'o=', 'd=', 'v', 'h'], ['config=', 'out=', 'output=', 'dst=', 'destination=', 'color=', 'crop=', 'verbose', 'help'])
			for opt, arg in opts:
				if opt in ('-c', '--config'):
					self.cfgPath = arg
				elif opt in ('-o', '-d', '--out', '--output', '--dst', '--destination'):
					self.outPath = arg
				elif opt in ('--color'):
					if arg in ('off', 'false'): self.doColMatch = False
				elif opt in ('--crop'):
					if arg in ('off', 'false'): self.doCrop = False
				elif opt in ('-v', '--verbose'):
					self.verbose = True
				elif opt in ('-h', '--help'):
					self.helpAndExit(0)
			self.imgPaths = self.rest
			
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		self.workstation = Workstation().fromConfig(json.load(open(self.cfgPath, 'r')))
		self.im = IMExec('/tmp', True)
		self.images = map(custImage, self.imgPaths)
		return self

	def execute(self):
		
		for image in self.images:
			
			# initialize
			if self.verbose: print "Convert source image (%s)" % image.path
			imTransParams, imColParams, imCropParams = '', '', ''
			pgmImg = self.im.convert(image, '', 'PGM')
			
			# transformation
			if self.verbose: print "Find the transformation matrix"
			transMatrix, scale = self.workstation.positionTransform(pgmImg)
			image.setTransform(Transform().initByMatrix(transMatrix))
			r = self.workstation.refTrans.r
			
			# ImageMagick output
			scale = (1.0, 1.0)
			a = [(r.calcPoint(Point(x, y)).asTuple(), (x * scale[0], y * scale[1])) for x in (0, self.workstation.refTrans.virtLayout.width) for y in (0, self.workstation.refTrans.virtLayout.height)]
			s = ' '.join(map(lambda x: ','.join(map(lambda y: str(round(y, 2)), x[0])) + ',' + ','.join(map(lambda y: str(round(y, 2)), x[1])), a))
			imTransParams = " -virtual-pixel black -interpolate NearestNeighbor +distort Perspective '%s'" % s
			
			if self.doColMatch:
				# color matching
				if self.verbose: print "Find the color transformation"
				ct = self.workstation.colorTransform(image)
				
				# ImageMagick output
				imColParams = ''
				for channel in ('RED', 'GREEN', 'BLUE'):
					poly = ct.getPolynom(channel).normPoints(255)
					imColParams += " -channel " + channel.lower() + " -fx '%s'" % str(poly)
					
					# gnuplot output
					print channel[0].lower() + '(u) = ' + poly.asString('**')
			
			# gnuplot output
			print "set xrange [0:1]"
			print "set yrange [0:1]"
			print "plot r(x), g(x), b(x)"
			
			if self.doCrop:
				# cropping
				if self.verbose: print "Cropping"
				cr = self.workstation.cropping(image)
				
				# ImageMagick output
				sTrans = Transform().initBySquare(Point(0, 0), scale)
				ct = cr.transform(sTrans)
				def printSigned(i):
					if i < 0:	return str(int(i))
					else:		return '+' + str(int(i))
				imCropParams += " -crop %ix%i%s%s" % (int(ct.width), int(ct.height), printSigned(ct.leftTop.x), printSigned(ct.leftTop.y))
			
			# Output
			self.imParams.append((image, imTransParams + imCropParams + imColParams))
		
		return self

	def printOut(self):
		
		for image, params in self.imParams:
			if self.debug or self.outPath == '': print params
			if self.outPath != '':
				if self.verbose: print "Converting to target"
				print params
				self.im.convertToDest(image, params, self.outPath)
		return self

	def helpAndExit(self, code=0):
		print "Transform the image according to the reference"
		print "with respect to color and distortion in 6 steps"
		print ""
		print "Usage: <cmd> [params] <file1> [<file2>]"
		print "-c --config		Workstation config file"
		print "-o --out --output	"
		print "-d --dst --destination	Output path"
		print "-v --verbose		Some additional output"
		print "--color			do Color matching"
		print "--crop			do Cropping"
		print "-h --help		This Help"
		exit(code)

# python metaScript.py --img /home/matthias/Desktop/ctest2/ex1/DSC_0415.JPG --ref /home/matthias/Desktop/ctest2/ex1/828056.png
if __name__ == '__main__':
	import sys, json
	o = App()
	o.run()
