
import include
import sys

from colorReference import colorReference
from Transform import Transform
from Shape import Point, Square
from custImage import custImage
from Dimension import Dimension
from Polynom import dPoly


class colMatch:
	def __init__(self, image, unique=True, invert=False):
		self.image = image
		self.polynom = None
		self.unique = unique
		self.invert = invert

	def matchReference(self, reference):
		points = []
		self.reference = reference
		for refCellPoint, imgCellPoint in self.reference:
			# get the reference color from the square in the reference image
			refColor = self.reference.getColorFromShape(refCellPoint)
			imgColor = self.image.getColorFromShape(imgCellPoint)
			
			# store color points
			if self.invert:	points.append(Point(imgColor, refColor))
			else:		points.append(Point(refColor, imgColor))
		
		if self.unique: points = self.__uniquePointlist(points)
		self.polynom = dPoly().initByPoints(points)
		return self.polynom

	def __uniquePointlist(self, input):
		uniqued = []
		for p0 in input:
			if not any([p0.x == p1.x and p0.y == p1.y and p0 != p1 for p1 in input]):
				uniqued.append(p0)
		return uniqued

	def __str__(self):
		if self.polynom == None: return "No matching yet for image '" + str(self.image.path) + "'"
		return str(self.polynom)


class App:
	#refPath
	#imgPath
	#geometry
	#offset
	#dimension
	#squareFac
	#channel
	#transData
	#invert
	#unique
	#output

	def __init__(self):
		self.transData = None
		self.output = sys.stdout
		self.unique = False
		self.invert = False
		#self.refPath = ''
		#self.imgPath = ''
		#self.geometry = None
		#self.dimension = None
		self.channel = 'value'
		self.degree = 2
		self.printCoeff = False

	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self


	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, rest = oParse.getopt(argv, ['r=','i=','g=','o=','d=','c=','p=','u','t=','h', 'k='], ['deg=', "reference", "image", "geometry", "offset", "dimension", "channel", "point", "invert", "unique", "target", "help"])
			for opt, arg in opts:
				if opt in ("-r", "--ref", "--reference"):
					self.refPath = arg
				elif opt in ('-i', '--image', '--img'):
					self.imgPath = arg
				elif opt in ('-k', '--deg'):
					self.degree = int(arg)
					self.printCoeff = True
				elif opt in ("-g", "--geometry"):
					geo = arg.split('x')
					self.geometry = Dimension(geo[0], geo[1])
					del geo
				elif opt in ("-o", "--offset"):
					off = arg.split('x')
					self.offset = Point(off[0], off[1])
					del off
				elif opt in ("-d", "--dimension"):
					dim = arg.split('x')
					self.dimension = Dimension(dim[0], dim[1])
					self.squareFac = float(dim[2])
					del dim
				elif opt in ("-c", "--channel"):
					self.channel = arg
				elif opt in ("-p", "--point"):
					point = arg.split('x')
					if self.transData == None:
						self.transData = Point(point[0], point[1])
					elif isinstance(self.transData, Point):
						self.transData = Square(self.transData, Point(point[0], point[1]))
					elif isinstance(self.transData, Square):
						self.transData.extend(Point(point[0], point[1]))
				elif opt == "--invert":
					self.invert = True
				elif opt in ("-u", "--unique"):
					self.unique = True
				elif opt in ("-t", "--target"):
					self.output = open(arg, 'w')
				elif opt in ("-h", "--help"):
					self.helpAndExit(0)

		except oParse.GetoptError:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		# reference
		self.reference = colorReference(self.refPath, self.channel, self.geometry, self.offset, self.dimension, self.squareFac)
		
		# transformation
		self.trans = Transform()
		if isinstance(self.transData, Square):
			scale = (float(self.transData.width)  / self.reference.dimension.width, float(self.transData.height) / self.reference.dimension.width)
			self.trans.initBySquare(self.transData.leftTop, scale)
		else:
			self.trans.initBySingle(self.transData)
		
		# Image
		self.image = custImage(self.imgPath, self.channel, self.trans)
		
		# color matching
		self.matching = colMatch(self.image, self.unique, self.invert)
		return self

	def execute(self):
		self.matching.matchReference(self.reference)
		return self

	def printOut(self):
		if self.printCoeff:
			self.output.write(str(self.matching.polynom.toCoeffPoly(self.degree)) + '\n')
		else:
			for pt in self.matching.polynom.points:
				self.output.write(str(pt.x) + "\t" + str(pt.y) + "\n")
		return self


	def helpAndExit(self, code=0):
		print "This script calculates the color correspondences of two images, given"
		print "the occurence of one in the other."
		print ""
		print "Usage:"
		print "Paths"
		print "-r --ref --reference	Reference path"
		print "-i --img --image	Image path"
		print ""
		print "Reference"
		print "-g --geometry		Columns and Rows (<Cols>x<Rows>)"
		print "-o --offset		Offset in the reference (<Left>x<Top>)"
		print "-d --dimension		Dimension of a cell in the reference image and the"
		print "			fraction of the used cell dimension"
		print "			(<Width>x<Height>x<Factor>)"
		print ""
		print "Probe"
		print "-c --channel		Channel name"
		print "-k --deg			Polynom degree for polynomial output"
		print "-p --point		Transformation point"
		print "			If one point is given, it has to be the coordinates"
		print "			of the upper left point of the reference."
		print "			If two points are given, the transformation"
		print "			will be scale-variant. More than two points are ignored"
		print ""
		print "Output"
		print "--invert		Inverts the order of the output columns"
		print "-o --output	output file (default stdout)"
		print ""
		print "Help"
		print "-h --help		This help"
		print ""
		print "The output consists of two columns, which represent the color in the"
		print "reference (left column) and in the image (right column). if --invert is"
		print "given, the order will be inversed"
		exit(code)



# python -i colMatch.py -i /home/matthias/Desktop/ctest2/ex1/OUT_0415_Final.jpg -r /home/matthias/Desktop/ctest2/ex1/828056.png -g 2x5 -o 120x0 -d 80x80x0.5 -c red -p 453x219 -p 1248x1323
if __name__ == '__main__': App().run()

#try:
	# init
	#m = colMatch()
	
	# Reference information
	#m.geometry = Dimension(2, 5)
	#m.offset = Point(120, 0)
	#m.dimension = Dimension(80, 80)
	
	# File information
	#m.imgPath = '/home/matthias/Desktop/ctest2/ex1/OUT_0415_Final.jpg'
	#m.refPath = '/home/matthias/Desktop/ctest2/ex1/828056.png'
	
	# Experiment information
	#m.channel = 'red'
	#m.squareFac = 0.5
	
	# Transformation information
	#m.transData = Square(Point(453, 219), Point(1248, 1323))
	
	# execute
	#m.initialize().execute().printOut()
	#m.run()
	
#except Exception, e:
#	print "There was an error while computing the color correspondences\n", e
#	exit(1)

