
import include
import sys, re

from Shape import Point, Square
from Transform import Transform
from Dimension import Dimension


"""
crop:
TERM	AS, AS

mCalc
AS	AS [+-] AS | (TS, TS) | [a-z][0-9]
TS	TS [+-] TS | [xyzc][0-3]\.[xy] | \d+
"""
class mCalc:
	def __init__(self, dimPix, dimMetric):
		# dimension in pix
		self.w0, self.h0 = dimPix
		# dimension in mm
		self.w1, self.h1 = self.__parseScalar(str(dimMetric[0]), 1.0), self.__parseScalar(str(dimMetric[1]), 1.0)
		# scale: pixel per mm
		self.scale = (float(self.w0)/self.w1, float(self.h0)/self.h1)
		
		self.fp = dict()
		self.fp['x0'] = Point(0, 0)
		self.fp['x1'] = Point(self.w0, 0)
		self.fp['x2'] = Point(self.w0, self.h0)
		self.fp['x3'] = Point(0, self.h0)

	def parse(self, s):
		return self.__readAS(s.strip().lower())

	def __readAS(self, s):
		#print "Input", s
		i = 0
		resPt = Point(0, 0)
		op = '+'
		while i < len(s):
			while i < len(s) and s[i] in (' ', '\t', '\n'): i+=1
			if i==len(s): break
			last = i
			
			if s[i] == '(':
				bcnt=0
				sep = 0
				i += 1
				while i < len(s):
					if s[i] == ')' and bcnt==0: break
					elif s[i] == ')': bcnt -= 1
					elif s[i] == '(': bcnt += 1
					elif s[i] == ',' and bcnt==0: sep = i
					i += 1
				curPt = self.__readTS(s[last+1:sep].strip(), s[sep+1:i].strip())
				while i < len(s) and s[i] not in ('+', '-'): i += 1
			else:
				while i < len(s) and s[i] not in ('+', '-'): i += 1
				curPt = self.__fixpoint(s[last:i].strip())
			
			if op == '-':	resPt -= curPt
			else:		resPt += curPt
			if i < len(s):	op = s[i]
			i += 1
		return resPt

	def __fixpoint(self, s):
		#print "fixpoint", s
		return self.fp[s]

	def __readTS(self, left, right):
		#print "read left", left, "and right", right
		x = self.__parseScalar(left, self.scale[0])
		y = self.__parseScalar(right, self.scale[1])
		return Point(x, y)

	def scalar(self, s):
		return self.__parseScalar(str(s), 1.0)

	def __parseScalar(self, s, scale):
		last = 0
		i=last
		resVal = 0
		if s[0] == '-' or s[0] == '+':
			op = s[0]
			i+=1
		else:
			op = '+'
		
		while i < len(s):
			
			# advance
			last = i
			while i < len(s) and s[i] not in ('+', '-'): i += 1
			
			# parse
			sprime = s[last:i].strip()
			m = re.match('(\d+(?:\.\d+)?)\s*(mm|cm|m)', sprime) 
			if m != None: # DM
				curVal = self.__normMetric(float(m.group(1)) * scale, m.group(2)) # DO SOME TRANSFORMATION
			elif re.match('\d+(?:\.\d+)?', sprime) != None: # DI
				curVal = float(sprime)
			elif re.match('[a-z][0-9]\.[xy]', sprime) != None: # FK
				p = sprime.split('.')
				if p[1] == 'x': curVal = self.__fixpoint(p[0]).x
				else: curVal = self.__fixpoint(p[0]).y
			else:
				raise Exception('cannot parse "' + sprime + '"')
			
			if op == '+':	resVal += curVal
			else:		resVal -= curVal
			if i < len(s): op = s[i]
			i += 1
		return resVal


	#
	# OLD METHODS
	#


	# Norm metric input to [mm]
	def __normMetric(self, val, metric):
		if metric == 'm':  val *= 1000
		if metric == 'dm': val *= 100
		if metric == 'cm': val *= 10
		return val

	# Return a fixpoint
	#def __fixpoint(self, val):
	#	# x, y = FP(val)
	#	return pt[1]

	def setFixpoint(self, index, point, prefix='y'):
		self.fp[prefix + str(index)] = point

	def setFixSquare(self, square, prefix='c'):
		self.fp[prefix + '0'] = square.leftTop
		self.fp[prefix + '1'] = square.rightTop
		self.fp[prefix + '2'] = square.rightBottom
		self.fp[prefix + '3'] = square.leftBottom
		#self.fp[prefix + '4'] = square.center	# center




class App:

	def __init__(self):
		self.result = []
		self.transData = None
		self.transform = None
		self.output = sys.stdout

	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self

	def parseArgs(self, argv):
		import oParse
		
		try:
			
			opts, self.request = oParse.getopt(argv, ['h', 'g=', 'd=', 'i=', 'p='], ['help', 'geometry=', 'dimension=', 'image=', 'point='])
			for opt, arg in opts:
				if opt in ("-h", "--help"):
					self.helpAndExit(0)
				elif opt in ("-g", "--geometry"):
					self.dimMetric = map(int, arg.split('x'))
				elif opt in ("-d", "--dimension"):
					self.dimPix = map(int, arg.split('x'))
				elif opt in ('-o', '--output'):
					self.output = open(arg, 'w')
				elif opt in ("-i", "--image"):
					img = image.custImage(arg)
					self.dimPix = (img.width, img.height)
				elif opt in ("-p", "--point"):
					point = map(int, arg.split('x'))
					if self.transData == None:
						self.transData = Point(point[0], point[1])
					elif isinstance(self.transData, Point):
						self.transData = Square(self.transData, Point(point[0], point[1]))
					elif isinstance(self.transData, Square):
						self.transData.extend(Point(point[0], point[1]))
			
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter: " + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		if self.transData != None:
			if isinstance(self.transData, Point):
				self.transform = Transform().initBySingle(self.transData)
			else:
				scale = (float(self.transData.width)  / self.dimPix[0], float(self.transData.height) / self.dimPix[1])
				self.transform = Transform().initBySquare(self.transData.leftTop, scale)
		self.calc = mCalc(self.dimPix, self.dimMetric)
		return self

	def execute(self):
		for req in self.request:
			curr = self.calc.parse(req)
			if self.transform != None: curr = curr.transform(self.transform)
			self.result.append((curr, req))
		return self

	def printOut(self):
		if len(self.result) > 1:
			for r, req in self.result:
				self.output.write(str(req) + '\t' + str(int(r)) + '\n')
		elif len(self.result) == 1:
			self.output.write(str(self.result[0][0]) + '\n')
		else:
			self.output.write('No data\n')
		return self


	def helpAndExit(self, code=0):
		print "This script calculates a pixel position, given a reference"
		print "and reference geometry"
		print ""
		print "Usage: [params] data1 data2 ..."
		print "Reference"
		print "-g --geometry		Image geometry in [mm]"
		print "-d --dimension		Image geometry in [pix]"
		print "-i --image		Image as path (will automatically ex -d)"
		print ""
		print "Transformation"
		print "-p --point		Transformation point"
		print "			There have to be at least 2 points given"
		print "Help"
		print "-h --help		This help"
		print ""
		print "Dataformat: [HV] / A sum of the following:"
		print " Fixpoints:	x0, x1, .."
		print " Metrix pos:	\d+ (mm|cm|m)"
		print " Pixel pos:	\d+"
		print ""
		print "The output will be a plain integer number, if only one"
		print "data input is given. Otherwise, it will be a two-column"
		print "table with the query on the left and the result on the right"
		exit(code)



# python -i mcalc.py -g 800x1143 -d 280x400 -p 453x219 -p 1248x1323 "V/x0 + 96" "H/x1 + 44"
if __name__ == '__main__': App().run()


"""
dimPix		= (280, 400)
dimMetric	= (800, 1143)
scale		= (1.5, 1.5)
offset		= (0, 0)
m = mCalc(dimPix, dimMetric, scale, offset)
print m.fromString('x1 + 5 mm - 4', True)
"""
