import sys, copy, include, os.path
from custImage import custImage
import ptMatch


class App:
	def __init__(self):
		self.output = sys.stdout
		self.doubleSize = False
	
	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self
	
	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['f=', 's=', 'd', 'o=', 'h'], ['help', 'imgA=', 'src=', 'dst=', 'source=', 'dest=', 'destination=', 'imgB=', 'doubleSize', 'detect', 'output='])
			for opt, arg in opts:
				if opt in ('-o', '--output'):
					self.output = open(arg, 'w')
				elif opt in ('-d', '--doubleSize'):
					self.doubleSize = True
				elif opt in ('-h', '--help'):
					self.helpAndExit(0)
			
			self.files = self.rest
			
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self
	
	def initialize(self):
		self.images = []
		for file in self.files:
			if not os.path.exists(file):
				print file, "does not exist"
				continue
			self.images.append(ptMatch.ptMatch(self.doubleSize, custImage(file)))
		return self
	
	def execute(self):
		self.matching = []
		for src in self.images:
			for trg in self.images:
				if src == trg: continue
				
				try:
					self.matching.append((src.image, trg.image, len(src.match(trg))))
					#self.matching.append((src.image, trg.image, src.match(trg)))
				except Exception, e:
					if str(e) != 'No Match': print e
					#self.matching.append((src.image, trg.image, []))
					self.matching.append((src.image, trg.image, 0))
		return self
	
	def printOut(self):
		for src, trg, match in self.matching:
			#self.output.write(os.path.basename(src.path) + "\t->" + os.path.basename(trg.path) + "\t:" + str(len(match)) + '\n')
			self.output.write(os.path.basename(src.path) + "\t->" + os.path.basename(trg.path) + "\t:" + str(match) + '\n')
		return self
	
	def helpAndExit(self, code=0):
		print ""
		print "Usage: <cmd> [params] <file1> <file2> ..."
		print "-o --output		Output file"
		print "-d --doubleSize		Double image size"
		print "-h --help		This Help"
		print ""
		exit(code)


if __name__ == '__main__': o = App().run()



