import sys, copy, include, os.path
from custImage import custImage
from Shape import Point, Square
import modMatch, modDetect

from iPoint import iPoint

class detectError(Exception): pass
class matchError(Exception): pass


class ptMatch:
	def __init__(self, doubleImageSize=False, image=None):
		self.doubleImageSize = doubleImageSize
		self.ipoints = None
		if image != None:	self.setImage(image)
		else:			self.image = None
	
	def setImage(self, image=None):
		if image != None: self.image = image
		if self.image.img.format.upper() not in ('PGM', 'PPM'): raise detectError('Image format must be PGM')
		try:
			self.ipoints = map(lambda ip: iPoint().fromRaw(ip), modDetect.detect(self.image.path, self.doubleImageSize))	# ipoints as objects
			#self.ipoints = modDetect.detect(self.image.path, self.doubleImageSize)
		except Exception, e: raise detectError(str(e))
	
	def setDoubleSize(self, doubleSize=True):
		self.doubleImageSize = doubleSize
		if self.image != None: self.setImage()
	
	def match(self, other):
		#return self.pyMatch(other)
		
		#if self.doubleImageSize != other.doubleImageSize: raise matchError('doubleImageSize differs')
		
		if self.ipoints == None: raise matchError('Not initialized')
		if other.ipoints == None: raise matchError('Not initialized')
		
		try:
			m = modMatch.match(map(lambda ip: ip.asRaw(), self.ipoints), map(lambda ip: ip.asRaw(), other.ipoints))
			if len(m) == 0: raise matchError('No Match')
			self.last = copy.deepcopy(m)
			return [(Point(fst[0], fst[1]), Point(snd[0], snd[1])) for fst, snd in m]
		except Exception, e: raise matchError(str(e))
		return self.__asPointList(self.last)

	def pyMatch(self, other):
		pts = []
		for ipA in self.ipoints:
			# get the distance to all ipoints of the other image
			dists = map(lambda ipB: (ipB, ipA.dist(ipB)), filter(lambda ipB: ipB.laplace == ipA.laplace, other.ipoints))
			match, mind = min(dists, key=lambda x: x[1])
			dists.remove((match, mind))
			second = min(dists, key=lambda x: x[1])
			
			if mind < 0.5 * second[1]: pts.append((ipA, match))
		
		if len(pts) == 0: raise matchError('No Match')
		return map(lambda ip: Point(ip.x, ip.y), pts)
	
	"""
	def pyMatch(self, other):
		pts = []
		for ipA in self.ipoints:
			mind, second = 1e100, 1e100
			match = None
			for ipB in other.ipoints:
				if ipA.laplace != ipB.laplace: continue
				d = ipA.dist(ipB)
				
				if d < mind:
					second = mind
					mind = d
					match = ipB
				elif d < second:
					second = d
		
			if mind < 0.5 * second:
				pts.append((ipA, match))
		
		if len(pts) == 0: raise matchError('No Match')
		return map(lambda ip: Point(ip.x, ip.y), pts)
	"""

	def __asPointList(self, inList):
		ptList = []
		for fst, snd in inList:
			ptList.append((Point(fst[0], fst[1]), Point(snd[0], snd[1])))
		return ptList

class App:
	def __init__(self):
		self.imageA = ptMatch(False)
		self.imageB = ptMatch(False)
		self.doMatch = True
		self.doubleSize = False
		self.output = sys.stdout
		self.fileA, self.fileB = '', ''
	
	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self
	
	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['f=', 's=', 'd', 'o=', 'h'], ['help', 'imgA=', 'src=', 'dst=', 'source=', 'dest=', 'destination=', 'imgB=', 'doubleSize', 'detect', 'output='])
			for opt, arg in opts:
				if opt in ('-f', '--imgA', '--src', '--source'):
					self.fileA = arg
				elif opt in ('-s', '--imgB', '--dst', '--dest', '--destination'):
					self.fileB = arg
				elif opt in ('--detect'):
					self.doMatch = False
				elif opt in ('-o', '--output'):
					self.output = open(arg, 'w')
				elif opt in ('-d', '--doubleSize'):
					self.doubleSize = True
				elif opt in ('-h', '--help'):
					self.helpAndExit(0)
			
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self
	
	def initialize(self):
		if self.doubleSize:
			self.imageA.setDoubleSize(self.doubleSize)
			self.imageB.setDoubleSize(self.doubleSize)
		if os.path.exists(self.fileA): self.imageA.setImage(custImage(self.fileA))
		if os.path.exists(self.fileB): self.imageB.setImage(custImage(self.fileB))
		return self
	
	def execute(self):
		if self.doMatch: self.ptList = self.imageA.match(self.imageB)
		return self
	
	def printOut(self):
		if self.doMatch:
			for src, dst in self.ptList:
				self.output.write(str(src) + '\t' + str(dst) + '\n')
		else:
			if self.imageA.ipoints != None:
				for item in self.imageA.ipoints: self.output.write(str(item) + '\n')
			if self.imageB.ipoints != None:
				for item in self.imageB.ipoints: self.output.write(str(item) + '\n')
		return self
	
	def helpAndExit(self, code=0):
		print ""
		print "Usage: <cmd> [params]"
		print "-f --imgA --src		Reference"
		print "-s --imgB --dst		Image"
		print "--detect			Detect only, no matching"
		print "-o --output		Output file"
		print "-d --doubleSize		Double image size"
		print "-h --help		This Help"
		print ""
		exit(code)


if __name__ == '__main__': o = App().run()



