
from Shape import Point, Square
from math import floor

class Line(Square):
	def __init__(self, region, step, width, direction='H', reverse=False):
		Square.__init__(self, region.leftTop, region.rightBottom)
		self.direction = direction
		self.step = step
		self.stepWidth = width
		self.reverse = reverse

	def factory(self, region):
		if self.direction == 'H':	direction = 'V'
		else:				direction = 'H'
		return Line(region, self.step, self.stepWidth, direction, self.reverse)

	def count(self):
		if self.direction == 'H':
			return floor(float(self.width) / self.step)
		else:
			return floor(float(self.height) / self.step)

	def norm(self):
		return (self.rightBottom - self.leftTop).norm()

	def __iter__(self):
		step = self.step
		if self.direction == 'H':
			if self.reverse:
				step *= -1
				x = self.rightBottom.x - self.stepWidth
				y = self.leftTop.y
				init = Square(Point(x, y), self.rightBottom)
			else:
				x = self.leftTop.x + self.stepWidth
				y = self.rightBottom.y
				init = Square(self.leftTop, Point(x, y))
			
			return HIterator(self, init, step, (self.leftTop.x, self.rightBottom.x))
			
		else: # self.direction == 'V'
			if self.reverse:
				step *= -1
				x = self.leftTop.x
				y = self.rightBottom.y - self.stepWidth
				init = Square(Point(x, y), self.rightBottom)
			else:
				x = self.rightBottom.x
				y = self.leftTop.y + self.stepWidth
				init = Square(self.leftTop, Point(x, y))
			
			return VIterator(self, init, step, (self.leftTop.y, self.rightBottom.y))

class LineIterator:
	def __init__(self, line, initial, step, range):
		self.line = line
		self.iterator = initial
		self.step = step
		self.min, self.max = range

	def __iter__(self):
		return self

	def next(self):
		lo, hi = self.move(self.iterator, self.step)
		if lo < self.min or hi > self.max:
			if self.min == 315: print "stop at", (hi, lo)
			raise StopIteration()
		return self.line.factory(self.iterator)


class HIterator(LineIterator):
	def __init__(self, line, initial, step, range):
		initial.leftTop.x -= step
		initial.rightBottom.x -= step
		LineIterator.__init__(self, line, initial, step, range)

	def move(self, it, step):
		it.leftTop.x += step
		it.rightBottom.x += step
		return (it.leftTop.x, it.rightBottom.x)


class VIterator(LineIterator):
	def __init__(self, line, initial, step, range):
		initial.leftTop.y -= step
		initial.rightBottom.y -= step
		LineIterator.__init__(self, line, initial, step, range)

	def move(self, it, step):
		it.leftTop.y += step
		it.rightBottom.y += step
		return (it.leftTop.y, it.rightBottom.y)


if __name__ == '__main__':
	final = None
	chances = 3
	
	#Scan Line: [(1310, 315), (2997, 1461)] V False
	#Scan Line: [(1272, 0), (2997, 1461)] H False
	
	line = Line(Square(Point(1310, 315), Point(2997, 500)), 1, 1, 'V', False)
	#line = Line(Square(Point(1272, 0), Point(2997, 1416)), 1, 1, 'H', False)
	elThres = 0.5
	
	for subLine in line:
		for shape in subLine:
			pass
	
	print final
