
from Shape import Point

class Matrix33:
	"""3x3 square matrix."""

	def __init__(self, a=0, b=0, c=0, d=0):
		self.matrix = []
		self.matrix.append([a, b, 1])
		self.matrix.append([c, d, 1])
		self.matrix.append([0, 0, 1])
		self.inverseMatrix = None

	def initByPoints(self, fst, snd, trd, frd):
		return self

	def initBySquare(self, s):
		self.matrix.append([s.width, 0, s.leftTop.x])
		self.matrix.append([0, s.height, s.leftTop.y])
		self.matrix.append([0, 0, 1])
		return self

	def invert(self):
		if self.inverseMatrix != None: return self.inverseMatrix
		self.inverseMatrix = Matrix33()
		self.inverseMatrix.inverseMatrix = self
		
		a, b, c = self.matrix[0]
		d, e, f = self.matrix[1]
		g, h, i = self.matrix[2]
		det = a*e*i + b*f*g + c*d*h - c*e*g - b*d*i - a*f*h
		self.inverseMatrix.matrix = [[e*i - f*h, c*h - b*i, b*f - c*e], [f*g - d*i, a*i - c*g, c*d - a*f], [d*h - e*g, b*g - a*h, a*e - b*d]]
		for i in range(3):
			for k in range(3):
				self.inverseMatrix.matrix[i][k] /= det
		return self.inverseMatrix

	def determinant(self):
		a, b, c = self.matrix[0]
		d, e, f = self.matrix[1]
		g, h, i = self.matrix[2]
		return a*e*i + b*f*g + c*d*h - c*e*g - b*d*i - a*f*h


	def transform(self, point):
		
		z = float(self.matrix[2][0] * point.x + self.matrix[2][1] * point.y + self.matrix[2][2])
		x = (self.matrix[0][0] * point.x + self.matrix[0][1] * point.y + self.matrix[0][2]) / z
		y = (self.matrix[1][0] * point.x + self.matrix[1][1] * point.y + self.matrix[1][2]) / z
		
		return Point(x, y)
	def calcPoint(self, point): return self.transform(point)

	def __str__(self):
		out = []
		for row in self.matrix:
			out.append('(' + ', '.join([str(col) for col in row]) + ')')
		return '[' + '; '.join(out) + ']'
