
from math import floor, ceil, sqrt

class Shape:
	def transform(self, trans): abstract()

class Point(Shape):
	"""Storage and routines for two dimensional coordinates."""

	def __init__(self, x, y, z=0.0):
		self.set(x, y, z)

	def set(self, x, y, z=0.0):
		self.x = float(x)
		self.y = float(y)
		self.z = float(z)
		return self

	def toSquare(self, dim, centre=True):
		"""Generate a square out of coordinates
		
		:param dim:	square dimensions
		:param centre:	if True, the square will be arount the point. Otherwise, the point is the upper left corner.
		"""
		if centre:
			leftTop		= Point(self.x - floor(dim.width/2), self.y - floor(dim.height/2), self.z)
			rightBottom	= Point(self.x + ceil(dim.width/2) , self.y + ceil(dim.height/2), self.z)
		else:
			leftTop		= Point(self.x            , self.y, self.z)
			rightBottom	= Point(self.x + dim.width, self.y + dim.height, self.z)
		return Square(leftTop, rightBottom)

	def asTuple(self):
		return (self.x, self.y)

	def asTriple(self):
		return (self.x, self.y, self.z)

	def __str__(self):
		return '(' + str(self.x) + ', ' + str(self.y) + ', ' + str(self.z) + ')'

	def transform(self, trans):
		"""Transforms itself with a given transformation handler."""
		return trans.transPoint(self)

	def norm(self):
		return sqrt(sum(map(lambda x: x**2, [self.x, self.y, self.z])))
	
	def __add__(self, other):
		return Point(self.x + other.x, self.y + other.y, self.z + other.z)
	
	def __sub__(self, other):
		return Point(self.x - other.x, self.y - other.y, self.z - other.z)

class Square(Shape):

	# Left top point as object of Coord
	# LeftTop

	# Right bottom point as object of Coord
	# RightBottom

	def __init__(self, first, second):
		self.leftTop		= Point(min(first.x, second.x), min(first.y, second.y))
		self.rightBottom	= Point(max(first.x, second.x), max(first.y, second.y))

	def toPoint(self):
		x = self.leftTop.x + floor(self.width/2.0)
		y = self.leftTop.y + floor(self.height/2.0)
		return Point(x, y)

	def extend(self, point):
		self.leftTop.set(min(self.leftTop.x, point.x), min(self.leftTop.y, point.y))
		self.rightBottom.set(max(self.rightBottom.x, point.x), max(self.rightBottom.y, point.y))

	def __getattr__(self, name):
		if name == 'leftBottom':
			return Point(self.leftTop.x, self.rightBottom.y)
		elif name == 'rightTop':
			return Point(self.rightBottom.x, self.leftTop.y)
		elif name == 'center':
			return self.leftTop + Point(floor(self.width/2.0), floor(self.height/2.0))
		elif name == 'height':
			return self.rightBottom.y - self.leftTop.y
		elif name == 'width':
			return self.rightBottom.x - self.leftTop.x
		raise AttributeError("instance has no attribute '" + str(name) + "'")
	
	def __setattr__(self, name, value):
		if name == 'width': raise Exception('width')
		if name == 'height': raise Exception('height')
		self.__dict__[name] = value
	
	def __str__(self):
		return '[' + str(self.leftTop) + ', ' + str(self.rightBottom) + ']'

	def transform(self, trans):
		return trans.transSquare(self)



if __name__ == '__main__':
	pass

