
from math import sqrt, floor
import Image
import os
from Shape import Point, Square

class custImage:

	# img, pix
	# channel
	# mean
	# width
	# height
	# transform

	def __init__(self, path, channel='value', trans=None, ctrans=None):
		self.channel = channel.upper()
		self.path = path
		
		# check, if the file exists and can be read
		if not os.path.exists(path) or not os.path.isfile(path):
			raise Exception('File does not exist or is not readable (%s)' % (path))
		
		self.transform = trans
		self.colTransform = ctrans
		self.img = Image.open(open(path, 'r'))
		self.pix = self.img.load()
		self.width, self.height = self.img.size

	def setTransform(self, trans):
		self.transform = trans

	def __setattr__(self, name, value):
		if name == 'channel': value = value.upper()
		self.__dict__[name] = value

	def getColorFromShape(self, shape):
		if isinstance(shape, Square):
			return self.getColorFromSquare(shape)
		elif isinstance(shape, Point):
			return self.getColorFromPoint(shape)
		else:
			raise Exception('Invalid shape')

	def getColorFromSquare(self, square):
		"""Calculates the mean color of a square area in an image."""
		
		# preparations
		colors = []
		point = Point(0, 0)
		
		# loop through the square
		for x in range(int(square.leftTop.x), int(square.rightBottom.x)):
			for y in range(int(square.leftTop.y), int(square.rightBottom.y)):
				
				# assign to point
				point.set(x, y)
				
				# get the color
				col = self.getColorFromPoint(point)
				colors.append(col)
		
		# return mean
		return self.euclid_mean(colors)

	def getColorFromPoint(self, point):
		"""Returns the color of an image at given coordinates."""
		
		# transform, if necessary
		if self.transform != None: point = point.transform(self.transform)
		
		# get the color from the image
		if point.x < 0 or point.y < 0 or point.x >= self.width or point.y >= self.height:
			rgb = (0, 0, 0)
			# warn
			#print "Index (" + str(point.x) + " , " + str(point.y) + ") out of range"
		else:
			rgb = self.pix[point.x, point.y]
		
		# return the value according to the channel
		if type(rgb) == type(0):
			value =  rgb
		elif self.channel == 'RED':
			value =  rgb[0]
		elif self.channel == 'GREEN':
			value =  rgb[1]
		elif self.channel == 'BLUE':
			value =  rgb[2]
		elif self.channel == 'SUM':
			value =  sum(rgb)
		elif self.channel == 'VALUE':
			if self.colTransform != None: return self.norm(self.colTransform.transRGB(rgb))
			value =  self.norm(rgb)
		elif self.channel == 'RGB':
			if self.colTransform != None: return self.colTransform.transRGB(rgb)
			return rgb
		else:
			raise Exception('Unknown channel ' + self.channel)
		
		if self.colTransform != None: value = self.colTransform.transValue(value, self.channel)
		return value

	def norm(self, input):
		return sqrt(sum([val**2 for val in input]))

	def euclid_mean(self, input):
		"""Calculates the euclidian mean."""
		
		# return zero on void input
		if len(input) == 0: return 0
		# calculate euclidian mean
		return float(sum(input)) / len(input)

	def __str__(self):
		raise Exception('Class Image cannot be converted into a string')



if __name__ == '__main__':
	pass

