
import include
from math import floor, ceil
from Dimension import Dimension
from custImage import custImage
from Shape import Point
from mCalc import mCalc

class colorReference:
	"""Collect color reference bar properties."""

	# rows: rows as int
	# cols: columns as int
	# offset: offset as object of coord
	# img: image resource
	# dim: {'ref': reference bar dimensions, 'cell': single colored cell dimensinos, 'square': comparative square dimensions}

	def __init__(self, path='', channel='value', geometry=None, offset=None, dim=None, fac=0.0, metric=None, size=('0','0')):
		"""
		
		It's adviced to use colorReference().fromConfig(cfgFile) for creating instances.
		"""
		self.metric = metric
		if path != '':
			self.rows = geometry.height
			self.cols = geometry.width
			self.offset = offset
			self.size = size
			
			self.dim = dict()
			self.dim['cell'] = dim
			self.dim['square'] = Dimension(floor(fac * dim.width), floor(fac * dim.height))
			self.dim['ref'] = Dimension(self.cols * dim.width + self.offset.x, self.rows * dim.height + self.offset.y)
			
			self.img = custImage(path, channel)
			
			if self.img.width < dim.width * geometry.width or self.img.height < dim.height * geometry.height:
				raise Exception('Image width or height not large enough (width = %i, %i expected / height = %i, %i expected' % (self.img.width, dim.width * geometry.width + offset.x, self.img.height, dim.height * geometry.height + offset.y))

	def __getattr__(self, name):
		if name == 'dimension':		return self.dim['ref']
		elif name == 'cell':		return self.dim['cell']
		elif name == 'square':		return self.dim['square']
		raise AttributeError()

	def __iter__(self):
		ptList = []
		
		m = mCalc((self.img.width, self.img.height), self.size)
		scale = [self.metric.scale[i] / m.scale[i] for i in (0,1)]
		
		for i in range(0, self.rows):
			for k in range(0, self.cols):
				
				# create Reference points
				# REPLACE WITH REFERENCE CLASS STRUCTURE
				x = k * self.cell.width  + ceil(float(self.cell.width) /2)
				y = i * self.cell.height + ceil(float(self.cell.height)/2)
				
				#print "Ref", Point(x, y), "\t-> ", (self.offset + Point(scale[0] * x, scale[1] * y))
				ptList.append((Point(x, y).toSquare(self.dim['square']), (self.offset + Point(scale[0] * x, scale[1] * y)).toSquare(self.dim['square'])))
		return ptList.__iter__()

	def fromConfig(self, cfg, metric=None):
		if metric==None: metric = self.metric
		dim = Dimension(cfg['dimension'][0], cfg['dimension'][1])
		geo = Dimension(cfg['geometry'][0], cfg['geometry'][1])
		size = str(cfg['size']).split(',')
		self.__init__(cfg['path'], cfg['channel'], geo, metric.parse(cfg['position']), dim, float(cfg['squareFac']), metric, size)
		self.polyDeg = int(cfg['polyDeg'])
		return self

	def getColorFromShape(self, shape):
		return self.img.getColorFromShape(shape)

	def __str__(self):
		raise Exception('Class Reference cannot be converted into a string')

