
import sys
sys.path.append('app')

import include, ptMatch, Ransac, colMatch, Crop, commands, os, random, string
from Dimension import Dimension
from custImage import custImage
from Shape import Point, Square
from Reference import Reference
from Transform import Transform
from ColorTrans import ColorTrans
from Polynom import dPoly
from mCalc2 import mCalc
from Crop2 import Crop

class IMExec:
	appPath = '/usr/bin/convert'
	def __init__(self, base, subdir=False):
		self.base = base
		if subdir:
			self.base = os.path.join(self.base, 'cira' + ''.join(random.Random().sample(string.letters + string.digits, 4)))
		os.makedirs(self.base)
	
	def tmpName(self, suffix):
		curr = os.path.join(self.base, '.')
		while os.path.exists(curr):
			tmpName = ''.join(random.Random().sample(string.letters + string.digits, 12)) + '.' + suffix
			curr = os.path.join(self.base, tmpName)
		return curr
	
	def convert(self, src, params, suffix='PNG'):
		return self.convertToDest(src, params, self.tmpName(suffix))

	def convertToDest(self, src, params, dst):
		cmd = self.appPath + ' ' + src.path + ' ' + params + ' ' + dst
		commands.getstatusoutput(cmd)
		return custImage(dst)

	def cleanUp(self):
		pass
		# rm self.base/*
		# rmdir self.base

class App:

	def __init__(self):
		# INTERNALS
		self.debug = False
		self.verbose = False
		self.doColMatch = True
		self.doCrop = True
		self.outPath = ''

		# OUTPUT VARIABLES
		self.imTransParams = ''
		self.imColParams = ''
		self.imCropParams = ''

		# FIXED PARAMETER
		self.refGeo = Dimension(2, 5)
		self.refDim = Dimension(80, 80)
		self.refOff = Point(120, 0)
		self.squareFac = 0.6
		self.polyDeg = 3;
		self.threshold = 10;
		self.cropping = [('bottom', ['F', 'x2 + (0, 52)']), ('left', ['S', '(x1.x + 10, 0), c2']), ('top', ['S', 'c0 + (0, x1.y), c2']), ('right', ['S', 'c0, c2'])]

	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self


	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['i=', 'r=', 'o=', 'd=', 't', 'm', 'c', 'v', 'h'], ['img=', 'image=', 'ref=', 'reference=', 'out=', 'output=', 'dst=', 'destination=', 'transform', 'color', 'crop', 'verbose', 'help'])
			for opt, arg in opts:
				if opt in ('-i', '--img', '--image'):
					self.srcPath = arg
				elif opt in ('-r', '--ref', '--reference'):
					self.refPath = arg
				elif opt in ('-o', '-d', '--out', '--output', '--dst', '--destination'):
					self.outPath = arg
				elif opt in ('-m', '--color'):
					if arg in ('off', 'false'): self.doColMatch = False
				elif opt in ('-c', '--crop'):
					if arg in ('off', 'false'): self.doCrop = False
				elif opt in ('-v', '--verbose'):
					self.verbose = True
				elif opt in ('-h', '--help'):
					self.helpAndExit(0)
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		self.srcImg = custImage(self.srcPath)
		self.refImg = custImage(self.refPath)
		self.reference = colorReference(self.refPath, 'value', self.refGeo, self.refOff, self.refDim, self.squareFac)
		self.im = IMExec('/tmp', True)
		return self

	def execute(self):
		# INITIALIZATION
		if self.verbose: print "Convert source images"
		transImg = self.im.convert(self.srcImg, '', 'PGM')
		transRef = self.im.convert(self.refImg, '', 'PGM')
		
		# IP MATCHING
		if self.verbose: print "Detect and match interesting points"
		matching = ptMatch.ptMatch(False, transRef).match(ptMatch.ptMatch(False, transImg))
		
		# TRANSFORMATION MATRIX
		if self.verbose: print "Find the transformation matrix"
		r = Ransac.Ransac(self.threshold)
		r.calcMatrix(matching)
		scale = r.getScale(self.refImg)
		offset = r.getOffset(self.srcImg)
		
		# ImageMagick output
		a = [(r.calcPoint(Point(x, y)).asTuple(), (x * scale[0], y * scale[1])) for x in (0, self.refImg.width) for y in (0, self.refImg.height)]
		s = ' '.join(map(lambda x: ','.join(map(lambda y: str(round(y, 2)), x[0])) + ',' + ','.join(map(lambda y: str(round(y, 2)), x[1])), a))
		self.imTransParams += " -virtual-pixel black -interpolate NearestNeighbor +distort Perspective '%s'" % s
		
		# set on image
		self.srcImg.setTransform(Transform().initByMatrix(r.transMatrix))
		
		if self.doColMatch:
			# COLOR MATCHING
			cTrans = ColorTrans()
			if self.verbose: sys.stdout.write("Find the color transformation: ")
			for channel in ['RED', 'GREEN', 'BLUE']:
				if self.verbose:
					sys.stdout.write(' ' + channel.lower())
					sys.stdout.flush()
				self.srcImg.channel = channel
				self.reference.img.channel = channel
				
				cm = colMatch.colMatch(self.srcImg, True, True)
				col = cm.matchReference(self.reference)
				cTrans.setTrans(channel, col.toCoeffPoly(self.polyDeg))

				# ImageMagick output
				imcol = dPoly().initByPoints(map(lambda p: Point(p.x/255, p.y/255), col.points))
				self.imColParams += " -channel " + channel.lower() + " -fx '%s'" % str(imcol.toCoeffPoly(self.polyDeg))
			if self.verbose: print ''

			self.srcImg.colTransform = cTrans

		if self.doCrop:
			if self.verbose: sys.stdout.write("Cropping: ")
			# preparations
			self.srcImg.channel = 'VALUE'
			metric = mCalc((self.refImg.width, self.refImg.height), (self.refImg.width, self.refImg.height))
			cr = Crop(self.srcImg, metric)
			
			# crop
			for border, args in self.cropping:
				if self.verbose:
					sys.stdout.write(' ' + border)
					sys.stdout.flush()
				cr.fromString(border, args)
			if self.verbose: print ""
			
			# ImageMagick output
			sTrans = Transform().initBySquare(Point(0, 0), scale)
			ct = cr.transform(sTrans)
			self.imCropParams += " -crop %ix%i+%i+%i" % (int(ct.width), int(ct.height), int(ct.leftTop.x), int(ct.leftTop.y))
		
		# TERMINATION
		return self

	def printOut(self):
		params = self.imTransParams + self.imCropParams + self.imColParams
		if self.debug: print params
		if self.outPath != '':
			if self.verbose: print "Converting to target"
			self.im.convertToDest(self.srcImg, params, self.outPath)
		else: print params
		return self

	def helpAndExit(self, code=0):
		print "Transform the image according to the reference"
		print "with respect to color and distortion in 6 steps"
		print ""
		print "Usage:"
		print "-i --img --image	Image path"
		print "-r --ref --reference	Reference path"
		print "-o --out --output	"
		print "-d --dst --destination	Output path"
		print "-v --verbose		Some additional output"
		print "-m --color		do Color matching"
		print "-c --color		do Cropping"
		print "-h --help		This Help"
		exit(code)

# python metaScript.py --img /home/matthias/Desktop/ctest2/ex1/DSC_0415.JPG --ref /home/matthias/Desktop/ctest2/ex1/828056.png
if __name__ == '__main__': App().run()
