
#include <Python.h>

#include <stdlib.h>
#include <math.h>

#include "match/match.h"
#include "match/match.c"

int min(int a, int b) {
	if (a < b) return a;
	return b;
}

void freeIpoints(Ipoint* ipts, int count) {
	int i;
	for (i=0; i<count; i++) {
		free(ipts[i].ivec);
	}
	free(ipts);
}

// [[mixed]]
Ipoint* loadIpoints(PyObject *inList, int *count, int *vlen) {
	
	// allocate resource
	*count = PyList_Size(inList);
	Ipoint *ipts = malloc(sizeof(Ipoint) * (*count));
	
	int n;
	for (n=0; n < *count; n++){
		PyObject *subList;
		subList = PyList_GetItem(inList, n);
		*vlen = PyList_Size(subList);
		
		// circular regions with diameter 5 x scale
		// Read in region data, though not needed for actual matching
		double x = PyFloat_AsDouble(PyList_GetItem(subList, 0));
		double y = PyFloat_AsDouble(PyList_GetItem(subList, 1));
		double a = PyFloat_AsDouble(PyList_GetItem(subList, 2));
		double b = PyFloat_AsDouble(PyList_GetItem(subList, 3));
		double c = PyFloat_AsDouble(PyList_GetItem(subList, 4));

		double det = sqrt((a-c)*(a-c) + 4.0*b*b);
		double e1 = 0.5*(a+c + det);
		double e2 = 0.5*(a+c - det);
		double l1 = (1.0/sqrt(e1));
		double l2 = (1.0/sqrt(e2));
		double sc = sqrt( l1*l2 );

		ipts[n].x = x;
		ipts[n].y = y;
		ipts[n].scale = sc/2.5;

		// Read in Laplacian
		ipts[n].laplace = (int)PyInt_AsLong(PyList_GetItem(subList, 5));

		// Strength & Orientation
		ipts[n].strength = PyFloat_AsDouble(PyList_GetItem(subList, 6));
		ipts[n].ori = PyFloat_AsDouble(PyList_GetItem(subList, 7));

		// SURF makes Laplacian part of descriptor, so skip it..
		
		ipts[n].ivec = malloc(sizeof(double) * (*vlen-1));
		unsigned int j;
		for (j = 0; j < *vlen - 1; j++) {
			ipts[n].ivec[j] = PyFloat_AsDouble(PyList_GetItem(subList, j+8));
		}

	}


	return ipts;
}

	int size1=0, size2=0, vlen1=0, vlen2=0;


int doMatch(Ipoint* ipts1, int size1, int vlen1, Ipoint* ipts2, int size2, int vlen2) {
	int vlen = min(vlen1, vlen2);
	p_list points[size1];
	int pcnt = findMatches(ipts1, ipts2, size1, size2, vlen, points);
	return pcnt;
}


// improve.match
static PyObject* improve_match(PyObject *self, PyObject *args) {
	Ipoint *ipts1, *ipts2;
	int i, size1=0, size2=0, vlen1=0, vlen2=0;
	
	// Read the arguments
	PyObject *fstList, *sndList;
	if (! PyArg_ParseTuple( args, "O!O!", &PyList_Type, &fstList, &PyList_Type, &sndList)) {
		PyErr_SetString(PyExc_RuntimeError, "Argument parse failure");
		return NULL;
	}

	ipts1 = loadIpoints(fstList, &size1, &vlen1);
	ipts2 = loadIpoints(sndList, &size2, &vlen2);

	int vlen = min(vlen1, vlen2);
	p_list points[size1];
	int pcnt = findMatches(ipts1, ipts2, size1, size2, vlen, points);

	PyObject *finList = PyList_New(0);
	for(i=0; i<pcnt; i++) {
		PyObject *tuple = PyTuple_New(2);
		PyObject *srcList = PyTuple_New(2);
		PyObject *dstList = PyTuple_New(2);
		int index1, index2;
		
		
		// Set vector contents
		index1 = points[i].first;
		index2 = points[i].second;
		PyTuple_SetItem(srcList, 0, PyFloat_FromDouble(ipts1[index1].x));
		PyTuple_SetItem(srcList, 1, PyFloat_FromDouble(ipts1[index1].y));
		PyTuple_SetItem(dstList, 0, PyFloat_FromDouble(ipts2[index2].x));
		PyTuple_SetItem(dstList, 1, PyFloat_FromDouble(ipts2[index2].y));

		// Set tuple contents
		PyTuple_SetItem(tuple, 0, srcList);
		PyTuple_SetItem(tuple, 1, dstList);
		
		// Append onto final list
		PyList_Append(finList, tuple);

	}
	
	// free resources
	freeIpoints(ipts1, size1);
	freeIpoints(ipts2, size2);
	
	return finList;
}



// /*

static PyMethodDef MatchMethods[] = {
	{"match",  improve_match, METH_VARARGS, "Some temp module"},
	{NULL, NULL, 0, NULL}        // Sentinel
};

PyMODINIT_FUNC initmodMatch(void) {
	(void) Py_InitModule("modMatch", MatchMethods);
}

// */
