
#include <Python.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polynom.h"


void printVector(double *in, char* name, int n) {
	int i;
	
	printf("%s = [ ", name);
	for(i=0; i < n; i++) {
		printf("%02.3f ", in[i]);
	}
	printf("]\n");
}


/**
 * 
 * ./polynom 
 * ./polynom -v -k <degree> -f <input file>
 * 
 */
static PyObject* modPolynom_Polynom(PyObject *self, PyObject *args) {
	
	double *X, *Y;
	int K = 0, N = 0, i;

	PyObject *inX, *inY;
	if (! PyArg_ParseTuple( args, "O!O!i", &PyList_Type, &inX, &PyList_Type, &inY, &K)) {
		PyErr_SetString(PyExc_RuntimeError, "Argument parse failure");
		return NULL;
	}
	
	N = PyList_Size(inX);
	if (N != PyList_Size(inY)) {
		PyErr_SetString(PyExc_RuntimeError, "sequence size differs");
		return NULL;
	}
	
	if (N == 0) {
		PyErr_SetString(PyExc_RuntimeError, "no data");
		return NULL;
	}
	
	X = malloc(sizeof(double) * N);
	Y = malloc(sizeof(double) * N);
	for(i=0; i<N; i++) {
		X[i] = PyFloat_AsDouble(PyList_GetItem(inX, i));
		Y[i] = PyFloat_AsDouble(PyList_GetItem(inY, i));
	}
	
	if (K > N-1) {
		K = N - 1;
	}
	
	if (K == 0) {
		PyErr_SetString(PyExc_RuntimeError, "polynom degree == 0");
		return NULL;
	}
	
	double *result = malloc(sizeof(double) * (K+1));
	poly(X, Y, N, K, result);
	
	
	PyObject *finList = PyList_New(0);
	for(i=0; i<K+1; i++) {
		PyList_Append(finList, PyFloat_FromDouble(result[i]));
	}
	return finList;
}




static PyMethodDef PolynomMethods[] = {
	{"polynom",  modPolynom_Polynom, METH_VARARGS, "Some temp module"},
	{NULL, NULL, 0, NULL}        // Sentinel
};


PyMODINIT_FUNC initmodPolynom(void) {
	(void) Py_InitModule("modPolynom", PolynomMethods);
}


