
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "ransac.h"

#ifdef CONTRACTS


#endif


/* INITIALIZATION */

void vecSetNull(vec in) {
	int i;
	
	for(i=0; i < DIM; i++) {
		in[i] = 0;
	}
}

void matrixSetNull(matrix in) {
	int i, k;
	
	for(i=0; i < DIM; i++) {
		for(k=0; k < DIM; k++) {
			in[i][k] = 0;
		}
	}
}

void matrixSetIdentity(matrix in) {
	int i;
	
	matrixSetNull(in);
	for(i=0; i < DIM; i++) {
		in[i][i] = 1;
	}
}





/* COPY FUNCTIONS*/

void copyVec(vec dst, vec src) {
	int i;
	
	for(i=0;i<DIM;i++) {
		dst[i] = src[i];
	}
}

void copyMatrix(matrix dst, matrix src) {
	int i, k;
	
	for(i=0; i < DIM; i++) {
		for(k=0; k < DIM; k++) {
			dst[i][k] = src[i][k];
		}
	}
}





/* COMPARISON FUNCTIONS */

bool cmpVec(vec a, vec b) {
	int i;
	
	for(i=0; i < DIM; i++) {
		if (a[i] != b[i]) {
			return FALSE;
		}
	}
	return TRUE;
}

/**
 * 
 * @return	Returns the array index on success, a negative number otherwise
 */
int in_array(void* haystack, int length, int el_size, void* needle) {
	byte* i = (byte*) haystack;
	byte* limit = i + length * el_size;
	
	for(; i < limit; i+=el_size) {
		if (memcmp(i, needle, el_size) == 0) {
			return ((i - (byte*)haystack) / el_size);
		}
	}
	return -1;
}


bool cmpArray(void* a, void* b, int el_size, int len) {
	byte* i = (byte*)a;
	byte* k = (byte*)b;
	byte* i_limit = i + len * el_size;
	byte* k_limit = k + len * el_size;
	bool found;
	
	for(; i < i_limit; i += el_size) {
		found = FALSE;
		for(k = (byte*)b; k < k_limit; k += el_size) {
			if (memcmp(i, k, el_size) == 0) {
				found = TRUE;
				break;
			}
		}
		
		if (found == FALSE) {
			return FALSE;
		}
	}
	return TRUE;
}






/* VECTOR AND MATRIX OPERATIONS */

/**
 * Calculates || a ||
 */
double euclid_norm(vec a) {
	int i;
	double sum = 0;
	
	for(i=0; i < DIM; i++) {
		sum += pow(a[i], 2);
	}
	return sqrt(sum);
}


/**
 * Calculates || a-b ||
 */

void subVec(vec a, vec b, vec c) {
	int i;
	
	for(i=0; i < DIM; i++) {
		c[i] = a[i] - b[i];
	}
}

double euclid_dist(vec a, vec b) {
	vec dist;
	
	subVec(a, b, dist);
	return euclid_norm(dist);
}

/**
 * Calculates 
 * y = H * x
 * H: Matrix (3x3)
 * x: Vector (3x1)
 */
void calcPoint(matrix model, vec point, vec image) {
	int i, k;
	
	// preparation
	vecSetNull(image);
	
	// simplified matrix product
	for(i=0; i < DIM; i++) {
		for(k=0; k < DIM; k++) {
			image[i] += point[k] * model[i][k];
		}
	}
	
	
	// normalize
	if (image[2] != 1.0) {
		for(i=0; i < DIM; i++) {
			image[i] /= image[2];
		}
	}
}


/* OUTPUT FUNCTIONS */

void printVector(vec in, char* name, bool newline, FILE* output) {
	int i;
	
	fprintf(output, "%s = [ ", name);
	for(i=0; i < DIM; i++) {
		fprintf(output, "%2.2f ", in[i]);
	}
	fprintf(output, "]");
	
	if (newline == TRUE) {
		fprintf(output, "\n");
	}
}

void printMatrix(matrix in, char* name, FILE* output) {
	int i, k, m, spaces;
	
	if (name[0] != '\0') {
		fprintf(output, "%s = ", name);
		spaces = strlen(name) + 2;
	} else {
		spaces = 0;
	}
	
	for(i=0; i < DIM; i++) {
		if (i >= 1) {
			for(m=0;m<spaces;m++) {
				fprintf(output, " ");
			}
		}
		fprintf(output, "[ ");
		for(k=0; k < DIM; k++) {
			fprintf(output, "%2.9f ", in[i][k]);
		}
		fprintf(output, "]\n");
	}
}

void printIntArray(int p[], int len, char* name) {
	int i;
	
	printf("%s = [ ", name);
	for(i=0;i<len;i++) {
		printf("%i ", p[i]);
	}
	printf("]\n");
}



/* UNUSED FUNCTIONS */

double maxVal(double a, double b) {
	if (a > b) return a;
	return b;
}

c_prec maxVec(vec a[], int n, short dim) {
	int i;
	c_prec max=0;
	
	for(i=0; i<n; i++) {
		if (a[i][dim] > max) {
			max = a[i][dim];
		}
	}
	
	return max;
}

