
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>

//#include <gandalf/vision.h>
#include <opencv/cv.h>

#include "ransac.h"
#include "model.h"


/*
bool model_caller(vec basis[], vec img[], int dim, matrix model) {
	
		int i;
		for(i=0; i<dim; i++) {
			//printf("[%f %f %f] => [%f %f %f]\n", basis[i][0], basis[i][1], basis[i][2], img[i][0], img[i][1], img[i][2]);
		}
		//if (dim == 4) {
			return cv_homography(basis, img, dim, model);
		//} else {
			return cv_homography(basis, img, dim, model);
		//}
}
*/

bool cv_homography(vec basis[], vec img[], int dim, matrix model) {
	CvMat* src;
	CvMat* dst;
	CvMat* homography;
	
	int i, k;
	
	// init values
	src = cvCreateMat(dim, 2, CV_32FC1);
	dst = cvCreateMat(dim, 2, CV_32FC1);
	
	for(i=0; i<dim; i++) {
		cvmSet(src,i,0,basis[i][0]);
		cvmSet(src,i,1,basis[i][1]);
		cvmSet(dst,i,0,img[i][0]);
		cvmSet(dst,i,1,img[i][1]);
	}
	
	homography = cvCreateMat(3, 3, CV_32FC1);
	
	// homography
	cvFindHomography(src, dst, homography, 0, 0, NULL /**/);
	
	// write back
	for(i=0; i<3; i++) {
		for(k=0; k<3; k++) {
			model[i][k] = cvmGet(homography,i,k);
		}
	}
	
	return TRUE;
}



/*
bool homography(vec basis[], vec img[], int dim, matrix model) {
	
	int i;
	Gan_Matrix33 m33P;

	// Version 1
	Gan_Homog33Match aMatch[dim];

	// Version 2
	Gan_Vector3 X[dim], x[dim];

	// Version 3
	Gan_SymMatEigenStruct SymEigen;

	// initialise eigensystem matrix //
	gan_homog33_init ( &SymEigen );
	int iEqCount=0;
	Gan_Vector3 v3X, v3x; // declare scene and image points X & x //


	for(i=0; i<dim; i++) {
		
		// Version 1
		// create and fill array of matches
		aMatch[i].type = GAN_HOMOG33_POINT;
		aMatch[i].d.p.X.x = basis[i][0];
		aMatch[i].d.p.X.y = basis[i][1];
		aMatch[i].d.p.X.z = 1.0;

		aMatch[i].d.p.x.x = img[i][0];
		aMatch[i].d.p.x.y = img[i][1];
		aMatch[i].d.p.x.z = 1.0;
		
		// Version 2
		x[i].x = basis[i][0];	x[i].y = basis[i][1];	x[i].z = 1.0;

		X[i].x = img[i][0];	X[i].y = img[i][1];	X[i].z = 1.0;

		// Version 3
		// ... build scene and image point coordinates into X and x ... //
		v3X.x = basis[i][0];	v3X.y = basis[i][1];	v3X.z = 1;
		v3x.x = img[i][0];	v3x.y = img[i][1];	v3x.z = 1;

		// increment matrix using point correspondence //
		gan_homog33_increment_p ( &SymEigen, &v3X, &v3x, 1.0, &iEqCount );
		
	}

	// fit projective 2D homography // 
	// Version 1
	if (gan_homog33_fit( aMatch, dim, &m33P ) == GAN_FALSE) return FALSE;
	
	// Version 2
	//if (gan_homog33_fit_points(x, X, dim, &m33P) == GAN_FALSE) return FALSE;
	
	
	// Version 3
	//gan_homog33_solve ( &SymEigen, iEqCount, &m33P );
	
	// return m33P
	//memcpy(model, m33P, sizeof(double) * 9);
	model[0][0] = m33P.xx;	model[0][1] = m33P.xy;	model[0][2] = m33P.xz;
	model[1][0] = m33P.yx;	model[1][1] = m33P.yy;	model[1][2] = m33P.yz;
	model[2][0] = m33P.zx;	model[2][1] = m33P.zy;	model[2][2] = m33P.zz;
	
	
	
	return TRUE;
}
*/





void gslPrintVector(gsl_vector* v, char *name) {
	int i;
	
	printf("%s = [ ", name);
	for(i=0; i<v->size; i++) {
		printf("%f ", gsl_vector_get(v, i));
	}
	printf("]\n");
}

void gslPrintMatrix(gsl_matrix *m, char *name) {
	int i, k, imax = m->size1, kmax = m->size2;
	
	printf("Matrix %s =\n", name);
	for(i=0; i<imax; i++) {
		printf("[ ");
		for(k=0; k<kmax; k++) {
			printf("%f ", gsl_matrix_get(m, i, k));
		}
		printf("]\n");
	}
}

void gslColToH(gsl_matrix *m, int column, matrix h) {
	
	int i=0, k=0, l=0;
	
	#ifdef DEBUG
		if (m->size1 != 9) {
			printf("ERROR!\n");
		}
	#endif
	
	l=0; k=0;
	for(i=0;i<m->size1;i++) {
		h[l][k] = gsl_matrix_get(m, i, column);
		printf("h[%i][%i] = %f\n", l, k, h[l][k]);
		if (k > 0 && k % (DIM-1) == 0) {
			l++; k=-1;
		}
		k++;
	}
}

/**
 * @todo	Replace LU-invert by matrix calculations
 */
gsl_matrix* gsl_invert(gsl_matrix *In) {
	gsl_matrix *inverse, *m;
	gsl_permutation *p;
	int sig;
	
	if (In->size1 != In->size2) {
		printf("Cannot invert non-quadratic matrix\n");
		return NULL;
	}
	
	// copy matrix
	m = gsl_matrix_alloc(In->size1, In->size2);
	gsl_matrix_memcpy(m, In);
	
	// LU
	p = gsl_permutation_alloc(m->size1);
	gsl_linalg_LU_decomp (m, p, &sig);
	
	// inverse
	inverse = gsl_matrix_alloc(m->size1, m->size1);
	gsl_linalg_LU_invert (m, p, inverse);
	
	// free resources
	gsl_permutation_free(p);
	gsl_matrix_free(m);
	return inverse;
}

/**
 * 
 * 
 * @param count		# Basisvektoren
 * @param basis		count Basisvektoren als vec[count][DIM] 
 * @param *model	Modellmatrix [DIM][DIM], wohin die Werte gespeichert werden müssen
 * 
 */
bool multReg(vec basis[], vec img[], int dim, matrix model) {
	
	gsl_matrix *X, *Xt, *XtX, *inverse, *block;
	gsl_vector *Y2, *T2, *Y1, *T1;
	int i, k;
	
	//printf("For %i input values: Calculated:\n", dim);
	
	// X [dim x 3]
	X = gsl_matrix_alloc(dim, 3);
	
	// Fill X
	for(i=0; i<dim; i++) {
		for(k=0; k<DIM; k++) {
			gsl_matrix_set(X, i, k, basis[i][k]);
		}
	}
	
	// Xt ([3 x dim])
	Xt = gsl_matrix_alloc(3, dim);
	gsl_matrix_transpose_memcpy(Xt, X);
	
	// Xt * X ([3 x 3])
	XtX = gsl_matrix_alloc(3, 3);
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, Xt, X, 0.0, XtX);
	
	// (Xt * X)^-1 ([3 x 3])
	inverse = gsl_invert(XtX);
	
	// (Xt * X)^-1 * Xt ([3 x dim])
	block = gsl_matrix_alloc(3, dim);
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inverse, Xt, 0.0, block);
	
	// Y1 ([dim x 1]), Y2 ([dim x 1])
	Y1 = gsl_vector_alloc(dim);
	Y2 = gsl_vector_alloc(dim);
	
	for(i=0; i<dim; i++) {
		gsl_vector_set(Y1, i, img[i][0]);
		gsl_vector_set(Y2, i, img[i][1]);
	}
	
	T1 = gsl_vector_alloc(3);
	T2 = gsl_vector_alloc(3);
	gsl_blas_dgemv(CblasNoTrans, 1.0, block, Y1, 0.0, T1);
	gsl_blas_dgemv(CblasNoTrans, 1.0, block, Y2, 0.0, T2);
	// Erste & Zweite Zeile von H
	for(i=0; i<3; i++) {
		model[0][i] = gsl_vector_get(T1, i);
		model[1][i] = gsl_vector_get(T2, i);
	}

	// Dritte Zeile von H
	model[2][0] = 0;
	model[2][1] = 0;
	model[2][2] = 1;

	//printMatrix(model, "MODEL");

	//exit(0);
	return TRUE;
}


gsl_matrix *transform(vec basis[], int dim) {
	
	gsl_matrix *Z = gsl_matrix_alloc(DIM, DIM);
	int i, k;
	
	for(i=0; i<DIM; i++) {
		for(k=0; k<DIM; k++) {
			gsl_matrix_set(Z, i, k, 0);
		}
	}
	
	for(i=0; i<DIM; i++) gsl_matrix_set(Z, i, i, 1);
	
	return Z;
	/*
	- 4 Punkte
	d(punkt - 0) = norm(punkt)
	
	
	H * punkt = [h11 * pkt(0) + h12 * pkt(1) | h21 * pkt(0) + h22 * pkt(1)]
	
	
	
	pkt = pkt1	norm(H * punkt) = sqrt((h11 * pkt(0) + h12 * pkt(1))^2 + (h21 * pkt(0) + h22 * pkt(1))^2)
	pkt = pkt1	norm(H * punkt) = sqrt((h11 * pkt(0) + h12 * pkt(1))^2 + (h21 * pkt(0) + h22 * pkt(1))^2)
	pkt = pkt1	norm(H * punkt) = sqrt((h11 * pkt(0) + h12 * pkt(1))^2 + (h21 * pkt(0) + h22 * pkt(1))^2)
	pkt = pkt1	norm(H * punkt) = sqrt((h11 * pkt(0) + h12 * pkt(1))^2 + (h21 * pkt(0) + h22 * pkt(1))^2)
	
	sum(norm)/4 = sqrt(2)
	* 
	
	sqrt()/4 + sqrt()/4 + sqrt()/4 + sqrt()/4 = sqrt(2)
	
	mittelwert (norm(H * punkt), für alle punkte) = sqrt(2)
	*/
	
}

gsl_matrix* matrixToGsl(matrix m) {
	gsl_matrix *Z = gsl_matrix_alloc(DIM, DIM);
	int i, k;
	
	for(i=0; i<DIM; i++) {
		for(k=0; k<DIM; k++) {
			gsl_matrix_set(Z, i, k, m[i][k]);
		}
	}
	
	return Z;
}

void gslToMatrix(gsl_matrix *Z, matrix m) {
	int i, k;
	for(i=0; i<DIM; i++) {
		for(k=0; k<DIM; k++) {
			m[i][k] = gsl_matrix_get(Z, i, k);
		}
	}
}

void gsl_calcPoint(gsl_matrix *m, vec src, vec dst) {
	int i, k;
	
	// preparation
	vecSetNull(dst);
	
	// simplified matrix product
	for(i=0; i < DIM; i++) {
		for(k=0; k < DIM; k++) {
			dst[i] += src[k] * gsl_matrix_get(m, i, k);
		}
	}
}

bool dltn(vec basis[], vec img[], int dim, matrix model) {
	/**
	 * Normalization of x
	 * 	Compute T
	 * Normalization of x'
	 * 	Compute T'
	 * DLT on x, x'
	 * 	Ht = DLT(xt, xst)
	 * Denormalization
	 * 	H = T'^-1 * Ht * T
	 * 
	 * 
	 * Transform computation
	 * - Tx = xt
	 *   xt: 0, 0 ist das Zentrum, im schnitt sqrt(2) abstand zum Mittelpunkt
	 * 
	 */
	
	gsl_matrix *T, *Ts, *Tsi, *H, *tmp;
	vec xt[dim], xst[dim];
	int i;
	
	// Transform basis, img
	T = transform(basis, dim);
	Ts = transform(img, dim);
	
	// get Xt, Xst
	for(i=0; i<dim; i++) {
		gsl_calcPoint(T, basis[i], xt[i]);
		gsl_calcPoint(Ts, img[i], xst[i]);
	}
	
	// DLT
	dlt(xt, xst, dim, model);
	
	// Denormalization
	H = matrixToGsl(model);
	Tsi = gsl_invert(Ts);
	
	// gTsi * H * T
	tmp = gsl_matrix_alloc(H->size1, T->size2);
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, H, T, 0.0, tmp);
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, tmp, Ts, 0.0, H);
	
	gslToMatrix(H, model);
	return TRUE;
}



bool dlt(vec basis[], vec img[], int dim, matrix model) {
	// not enough points
	if (dim < 4) return FALSE;
	
	/**
	 * Given n>= 4 2D - 2D point corr. (xi <-> xi'), determine H, such that xi' = Hxi
	 * 
	 * (1)	For each corr. xi <-> xi', compute matrix Ai
	 * 	Only the first two rows needed in general
	 * (2)	Assemble the n 2x9 matrices Ai into a single 2nx9 matrix A
	 * (3)	Obtain the SVD of A
	 * 	The unit singular vector corresponding to the smallest singular value is the solution h
	 * 	Specifically, if A = UDVt with D diagonal with positive diagonal entries, arranged in descending order
	 * 	down the diagonal, then h is the last column of V
	 * (4)	The matrix H is determined from h 
	 */
	
	/**
	 * wi' = img[i][2]
	 * yi' = img[i][1]
	 * xi' = img[i][0]
	 * 
	 * xi1 = basis[i][0]
	 * xi2 = basis[i][1]
	 * xi3 = basis[i][2]
	 */
	
	gsl_matrix *U, *Input, *V;
	gsl_vector *S, *work;
	double w, y, x;
	int i, k;
	
	
	for(i=0; i<dim; i++) {
		basis[i][2] = 1;
		img[i][2] = 1;
		printVector(basis[i], "SRC", FALSE, stdout);
		printf("\t");
		printVector(img[i], "IMG", TRUE, stdout);
	}
	
	// Assemble Input matrix (2n x 9 - Matrix)
	Input = gsl_matrix_alloc(2 * dim, 9);		// M x N
	k=0;
	for(i=0; i<dim; i++) {
		w = img[i][2]; y = img[i][1]; x = img[i][0];
		
		printVector(basis[i], "BASIS", FALSE, stdout);
		printf("\t");
		printVector(img[i], "IMG", TRUE, stdout);
		
		// Input[k]
		gsl_matrix_set(Input, k, 0, 0);		gsl_matrix_set(Input, k, 3, -w * basis[i][0]);		gsl_matrix_set(Input, k, 6, y * basis[i][0]);
		gsl_matrix_set(Input, k, 1, 0);		gsl_matrix_set(Input, k, 4, -w * basis[i][1]);		gsl_matrix_set(Input, k, 7, y * basis[i][1]);
		gsl_matrix_set(Input, k, 2, 0);		gsl_matrix_set(Input, k, 5, -w * basis[i][2]);		gsl_matrix_set(Input, k, 8, y * basis[i][2]);
		
		// Input[k+1]
		k++;
		gsl_matrix_set(Input, k, 0, w * basis[i][0]);		gsl_matrix_set(Input, k, 3, 0);		gsl_matrix_set(Input, k, 6, -x * basis[i][0]);
		gsl_matrix_set(Input, k, 1, w * basis[i][1]);		gsl_matrix_set(Input, k, 4, 0);		gsl_matrix_set(Input, k, 7, -x * basis[i][0]);
		gsl_matrix_set(Input, k, 2, w * basis[i][2]);		gsl_matrix_set(Input, k, 5, 0);		gsl_matrix_set(Input, k, 8, -x * basis[i][0]);
		
		k++;
	}
	
	printf("2 * dim = %i, k = %i\n", 2*dim, k);
	gslPrintMatrix(Input, "INPUT");
	
	// M<N not implementet, therefore transpose, if 2n < 9
	k = 2*dim;
	if (dim < 5) {
		
		printf("Transpose\n");
		// Transpose
		U = gsl_matrix_alloc(9, k);
		gsl_matrix_transpose_memcpy(U, Input);
		
		// Allocate work matrices and vectors
		V = gsl_matrix_alloc(k, k);
		S = gsl_vector_alloc(k);
		work = gsl_vector_alloc(k);
	} else {
		
		U = Input;
		
		// Allocate work matrices and vectors
		V = gsl_matrix_alloc(9, 9);
		S = gsl_vector_alloc(9);
		work = gsl_vector_alloc(9);
	}
	
	// Obtain SVD
	if (gsl_linalg_SV_decomp(U, V, S, work) == GSL_SUCCESS) {
		
		gslPrintMatrix(U, "U");
		gslPrintMatrix(V, "V");
		gslPrintVector(S, "S");
		
		// Extract H
		if (dim < 5) {
			// H is the last column of U
			gslColToH(U, U->size2 - 1, model);
		} else {
			// H is the last column of V
			gslColToH(V, V->size2 - 1, model);
		}
	} else {
		// ERROR: NO SVD
		return FALSE;
	}
	
	
	printMatrix(model, "MODEL", stdout);

	// invert model
	gsl_matrix *h = gsl_matrix_alloc(3, 3), *inverse;
	
	for(i=0; i<3; i++) {
		for(k=0; k<3; k++) {
			gsl_matrix_set(h, i, k, model[i][k]);
		}
	}
	
	inverse = gsl_invert(h);
	
	for(i=0; i<3; i++) {
		for(k=0; k<3; k++) {
			model[i][k] = gsl_matrix_get(inverse, i, k);
		}
	}
	
	printMatrix(model, "INVERSE", stdout);
	
	// normalize h
	/*
	for(i=0; i<DIM; i++) {
		for(k=0; k<DIM; k++) {
			model[i][k] /= model[2][2];
		}
	}
	*/
	// Build up H
	
	// clean up
	/*
	gsl_matrix_free(U);
	gsl_matrix_free(Input);
	gsl_matrix_free(V);
	gsl_vector_free(S);
	gsl_vector_free(work);
	*/
	return TRUE;
}


