
/**
 * DEFINE
 */

#define DIM 3


/**
 * TYPEDEF
 */

typedef double c_prec;

typedef char byte;
typedef unsigned char u_byte;

typedef c_prec vec[DIM];

typedef c_prec matrix[DIM][DIM]; 

typedef double q_prec;

typedef enum {TRUE = 1, FALSE = 0} bool;

typedef struct {
	int set;
	int loops;
	double e;
	double dist;
} r_param;

typedef struct {
	vec *src;
	vec *img;
	int dim;
} r_data;

typedef struct {
	double set, out, tot;
} t_quality;

typedef struct {
	double x, y;
} t_curve;


typedef struct {
	double mean, sum, min, max, var;
} t_stats;




/**
 * FUNCTIONS
 */


/** RANSAC */
double ransac(r_data *data, r_param *params, int mp_cnt, matrix model_final);
bool setToModel(r_data *data, int set[], int len, matrix model);


/** RANSAC EVALUATION */

// STATISTICAL DATA
t_stats data_dist_stats(r_data *data);
t_stats curve_stats(t_curve curve[], int length);

// PARAMETER APPROXIMATION
int approx_dist_qcurve(r_data *data, r_param *params, int mp_cnt, int max, double step, t_curve curve[]);
bool approx_set_and_loop(r_data *data, r_param *params, int mp_cnt);

t_curve approx_dist_dyn_max(r_data *data, r_param *params, int mp_cnt, double step, int width, int chances);
int approx_set_curve(r_data *data, r_param *params, int mp_cnt, double e, int max, double step, t_curve curve[]);
int approx_loop_curve(r_data *data, r_param *params, int mp_cnt, double e, int max, double step, t_curve curve[]);

// PARAMETER
void calcParams(r_param *params, double e, int n, int m);

// MEASUREMENT
t_quality calcQuality(matrix model, r_data *data, r_param *params);
q_prec calcMeanDist(matrix model, r_data *data);

// OUTPUT
void evalModel(r_data *data, r_param *params, matrix model, double m_cnt, bool stat, FILE* output);





/** BASICS */

// INITIALIZATION
void vecSetNull(vec in);
void matrixSetNull(matrix in);
void matrixSetIdentity(matrix in);

// COPY FUNCTIONS
void copyVec(vec dst, vec src);
void copyMatrix(matrix dst, matrix src);

// COMPARISON FUNCTIONS
bool cmpVec(vec a, vec b);
int in_array(void* haystack, int length, int el_size, void* needle);
bool cmpArray(void* a, void* b, int el_size, int len);

// VECTOR AND MATRIX OPERATIONS
double euclid_norm(vec a);
void subVec(vec a, vec b, vec c);
double euclid_dist(vec a, vec b);
void calcPoint(matrix model, vec point, vec image);

// OUTPUT FUNCTIONS
void printVector(vec in, char* name, bool newline, FILE* output);
void printMatrix(matrix in, char* name, FILE* output);
void printIntArray(int p[], int len, char* name);

// UNUSED FUNCTIONS
double maxVal(double a, double b);
c_prec maxVec(vec a[], int n, short dim);

