
import sys
sys.path.append('/home/matthias/projekte/cira/software/implementation/kommando/pyImp/app')

import include, ptMatch, Ransac, colMatch, Crop, commands, os, random, string
from Dimension import Dimension
from custImage import custImage
from Shape import Point, Square
from Reference import Reference
from Transform import Transform
from ColorTrans import ColorTrans
from Polynom import dPoly
from mCalc2 import mCalc
from Crop2 import Crop


"""
Input (JSON)
- Referenzen
- Farbfelder
- Geometrie der Referenzen
  > Cropping-Grenzen (nutze mCalc)
  > Farbefeld-Position (nutze mCalc)

- Definition von Fixpunkten (für mCalc)
  > Config oder Hardcoded (!?)
- Finden der Transformationspunkte (1/#Referenzen Punkte pro Referenz)
  > Referenzen und deren Geometrie aus Input
  > Skalierung und Offset
- Abgleichen der Farben
  > Angaben der Geometrie aus Input
- Cropping
  > Angaben der Geometrie aus Input

Output
- Indirekt:	Schnittstellen für die einzelnen Schritte
- Direkt:	Wie metaScript.py



metaScript:
- image
- reference (reference + colorPanel)
- output

- ip, transform
	self.refGeo		= Dimension(2, 5)
	self.refDim		= Dimension(80, 80)
	self.refOff		= Point(120, 0)
	self.squareFac		= 0.6
	self.polyDeg		= 3;
	self.threshold		= 10;
	self.cropping		= [('bottom', ['F', 'x2 + (0, 52)']), ('left', ['S', '(x1.x + 10, 0), c2']), ('top', ['S', 'c0 + (0, x1.y), c2']), ('right', ['S', 'c0, c2'])]
- color matching, cropping


Workstation:
- Container für
	references:	Bildpfade
	colorPanel:	Bildpfad, Position, relativ zu Referenzpunkten
	cropping:	Grenzen, relativ zu Referenzpunkten
- transformieren
	=> Verschiedene Algorithmen in abgeleiteten klassen
		I
			a) Finde Punkte von N Referenzen => N Transformationsmatrizen
			b) Für jeden Referenzpunkt, berechne Mittelpunkt
			c) 1 Transformationsmatrix aus allen Mittelpunkten
		II
			a) Finde Punkte von N Referenzen
			b) Transformationsmatrix aus allen Punkten
		=> Relative Position der Referenzen gegeben (mCalc ohne Fixpunkte...)
		   => Eckpunkte der Referenzen für mCalc

metaScript II:
- Workstation => Config
- image
- output
> Init ColTransform, Cropping, ...
> Transform input (PGM)
> Workstation -> transformation
> ColTransform -> transformation (Workstation)
> Cropping -> transformieren (Workstation)

Parameter
- (relative) Position, Art der Referenzen
- (relative) Position, Art der Farbstreifen
- Bilder der Referenzen, Farbstreifen; Alg. Parameter zu Referenzen, Farbstreifen
- Cropping

Anforderungen
- Falls nur eine Referenz angegeben wurde, muss sich die Workstation gleich verhalten wie bisher

Eingaben
- Bild

Ausgaben
- Transformationsmatrix
- Farbpolynom für jeden Kanal
- Cropping

"""

from IMExec import IMExec

class Workstation:
	def __init__(self):
		pass

	def fromConfig(self, cfg):
		self.refTrans = multiRef.multiRef().fromConfig(cfg)
		self.metric = self.refTrans.virtLayout.metric
		self.colTrans = colorReference.colorReference().fromConfig(cfg, self.metric)
		self.cropping = cfg['cropping']
		pass

	def positionTransform(self, image):
		# get transformation
		transMatrix, scale = self.refTrans.multiRef(image)
		
		# set on image
		image.setTransform(Transform().initByMatrix(transMatrix))
		return scale

	def colorTransform(self, image):
		cTrans = ColorTrans()
		for channel in ['RED', 'GREEN', 'BLUE']:
			image.channel = channel
			self.colTrans.img.channel = channel
			
			cm = colMatch.colMatch(image, True, True)
			col = cm.matchReference(self.colTrans)
			cTrans.setTrans(channel, col.toCoeffPoly(self.colTrans.polyDeg))
		
		image.colTransform = cTrans
		return cTrans

	def cropping(self, image, metric=None):
		if metric == None: metric = self.metric
		
		image.channel = 'VALUE'
		cr = Crop(image, metric)
		
		# crop
		for border, args in self.cropping:
			cr.fromString(border, args)
		
		return cr

class App:

	def __init__(self):
		# INTERNALS
		self.debug = False
		self.verbose = False
		self.doColMatch = True
		self.doCrop = True
		self.outPath = ''

		# OUTPUT VARIABLES
		self.imParams = []


	def run(self):
		self.parseArgs(sys.argv[1:])
		self.initialize()
		self.execute()
		self.printOut()
		return self


	def parseArgs(self, argv):
		import oParse
		
		try:
			opts, self.rest = oParse.getopt(argv, ['c=', 'o=', 'd=', 'm=', 'c=', 'v', 'h'], ['config=', 'out=', 'output=', 'dst=', 'destination=', 'color=', 'crop=', 'verbose', 'help'])
			for opt, arg in opts:
				if opt in ('-c', '--config'):
					self.cfgPath = arg
				elif opt in ('-o', '-d', '--out', '--output', '--dst', '--destination'):
					self.outPath = arg
				elif opt in ('-m', '--color'):
					if arg in ('off', 'false'): self.doColMatch = False
				elif opt in ('-c', '--crop'):
					if arg in ('off', 'false'): self.doCrop = False
				elif opt in ('-v', '--verbose'):
					self.verbose = True
				elif opt in ('-h', '--help'):
					self.helpAndExit(0)
			self.imgPaths = rest
		except oParse.GetoptError, e:
			sys.stderr.write("An error was detected while parsing parameter" + str(e) + '\n')
			self.helpAndExit(1)
		
		return self

	def initialize(self):
		self.workstation = Workstation().fromConfig(open(self.cfgPath, 'r'))
		self.im = IMExec('/tmp', True)
		self.images = map(custImage, self.imgPaths)
		return self

	def execute(self):
		
		for origImg in self.images:
			
			# initialize
			imTransParams, imColParams, imCropParams = '', '', ''
			image = self.im.convert(origImg, '', 'PGM')
			
			# transformation
			scale = self.workstation.positionTransform(image)
			r = self.workstation.refTrans.r
			
			# ImageMagick output
			a = [(r.calcPoint(Point(x, y)).asTuple(), (x * scale[0], y * scale[1])) for x in (0, self.workstation.refTrans.virtLayout.width) for y in (0, self.workstation.refTrans.virtLayout.height)]
			s = ' '.join(map(lambda x: ','.join(map(lambda y: str(round(y, 2)), x[0])) + ',' + ','.join(map(lambda y: str(round(y, 2)), x[1])), a))
			imTransParams = " -virtual-pixel black -interpolate NearestNeighbor +distort Perspective '%s'" % s
			
			if self.doColMatch:
				# color matching
				ct = self.workstation.colorTransform(image)
				
				# ImageMagick output
				imColParams = ''
				for channel in ('RED', 'GREEN', 'BLUE'):
					poly = ct.getPolynom(channel).normPoints(255)
					imColParams += " -channel " + channel.lower() + " -fx '%s'" % str(poly)
			
			if self.doCrop:
				# cropping
				cr = self.workstation.cropping(image)
				
				# ImageMagick output
				sTrans = Transform().initBySquare(Point(0, 0), scale)
				ct = cr.transform(sTrans)
				imCropParams += " -crop %ix%i+%i+%i" % (int(ct.width), int(ct.height), int(ct.leftTop.x), int(ct.leftTop.y))
			
			# Output
			self.imParams.append((origImg, imTransParams + imCropParams + imColParams))
		
		return self

	def printOut(self):
		
		for image, params in self.imParams:
			if self.debug or self.outPath == '': print params
			if self.outPath != '':
				if self.verbose: print "Converting to target"
				self.im.convertToDest(image, params, self.outPath)
		return self

	def helpAndExit(self, code=0):
		print "Transform the image according to the reference"
		print "with respect to color and distortion in 6 steps"
		print ""
		print "Usage: <cmd> [params] <file1> [<file2>]"
		print "-c --config		Workstation config file"
		print "-o --out --output	"
		print "-d --dst --destination	Output path"
		print "-v --verbose		Some additional output"
		print "-m --color		do Color matching"
		print "-c --color		do Cropping"
		print "-h --help		This Help"
		exit(code)

# python metaScript.py --img /home/matthias/Desktop/ctest2/ex1/DSC_0415.JPG --ref /home/matthias/Desktop/ctest2/ex1/828056.png
if __name__ == '__main__': App().run()
